@@ -589,7 +589,9 @@ class ExpectDataType(Enum):
     rgb = 'rgb'
     rgba = 'rgba'
     label_mask = 'label_mask'
+    color_mask = 'color_mask'
     binary_mask = 'binary_mask'
+    alpha_mask = 'alpha_mask'
     multi_channel = 'multi_channel'
     absolute_bbox='absolute_bbox'
     relative_bbox='relative_bbox'
