@@ -31,7 +31,7 @@ import numpy as np
 from .common import to_list,addindent,camel2snake,snake2camel,unpack_singleton,enforce_singleton,OrderedDict
 from .pytorch_ops import *
 
-__all__ = ['to_numpy','to_tensor','print_network','plot_tensor_grid','summary','calculate_flops', 'Layer', 'Sequential', 'Input', 'get_device', 'load','Combine','ReplayBuffer','check_keys','try_map_args_and_call','get_signature']
+__all__ = ['get_uid','to_numpy','to_tensor','print_network','plot_tensor_grid','summary','calculate_flops', 'Layer', 'Sequential', 'Input', 'get_device', 'load','Combine','ReplayBuffer','check_keys','try_map_args_and_call','get_signature']
 
 version=torch.__version__
 sys.stderr.write('Pytorch version:{0}.\n'.format(version))
@@ -85,11 +85,15 @@ def to_numpy(x) -> np.ndarray:
 
 def to_tensor(x, dtype=torch.float32,requires_grad=None) -> torch.Tensor:
     if isinstance(x,  torch.Tensor):
+        x = x.clone().detach()
+        x = x.to(_device)
         if dtype is not None:
             x = x.type(dtype)
-        x=x.to(_device)
-        if requires_grad is not None:
-            x.requires_grad=requires_grad
+        if requires_grad ==False:
+            x.requires_grad =False
+        elif requires_grad ==True:
+            x.requires_grad=True
+
         return x
     elif isinstance(x, int):
         return torch.tensor(x).int().to(_device) if requires_grad is None else torch.tensor(x,requires_grad=requires_grad).int().to(_device)
@@ -104,12 +108,16 @@ def to_tensor(x, dtype=torch.float32,requires_grad=None) -> torch.Tensor:
         return x
     elif isinstance(x, np.ndarray):
         npdtype=x.dtype
-        x = torch.tensor(x) if requires_grad is None else torch.tensor(x,requires_grad=requires_grad)
+        x = torch.tensor(x)
         if 'int' in str(npdtype):
             x = x.type(torch.int64)
         else:
             x = x.type(dtype)
         x = x.to(_device)
+        if requires_grad == False:
+            x.requires_grad = False
+        elif requires_grad == True:
+            x.requires_grad = True
         return x
     else:
         raise ValueError("Unsupported input type" + str(type(x)))
@@ -159,11 +167,11 @@ def get_uid(prefix=''):
 #     return 'InputSpec(%s)' % ', '.join(x for x in spec if x)
 
 
+
+
 class Layer(nn.Module):
     """
-    Applies a lambda function on forward()
-    Args:
-        lamb (fn): the lambda function
+    Trident extened nn.Module
     """
 
     def __init__(self,**kwargs):
@@ -175,8 +183,8 @@ class Layer(nn.Module):
         self._input_shape = None
         self._output_shape = None
 
-        self._output_tensor = None
-        self.is_track_output=False
+        self._output_tensor =None
+        self.keep_output=False
         self.signature=None
 
         # self._input_shape=None
@@ -188,9 +196,8 @@ class Layer(nn.Module):
         self.defaultname = camel2snake(prefix) + '_' + str(get_uid(prefix))
 
         self._name = kwargs.get('name')
-        self.dump_patches = True
+        #self.dump_patches = True
 
-        self.register_buffer('uuid',torch.tensor(uuid.uuid4().node))
         self.uuid=torch.tensor(uuid.uuid4().node)
 
         self._nodes = OrderedDict()
@@ -295,7 +302,7 @@ class Layer(nn.Module):
         return list(self.parameters())
 
     def get_weights(self):
-        return list(self.parameters())
+        return [p.numpy() for p in list(self.parameters())]
 
 
     def set_weights(self, weights):
@@ -385,19 +392,6 @@ class Layer(nn.Module):
     #     """
     #     return self._input_tensors
 
-    @property
-    def output(self):
-        """Retrieves the output tensor(s) of a layer.
-        Only applicable if the layer has exactly one output,
-        i.e. if it is connected to one incoming layer.
-        Returns:
-          Output tensor or list of output tensors.
-        Raises:
-          AttributeError: if the layer is connected to more than one incoming
-            layers.
-          RuntimeError: if called in Eager mode.
-        """
-        return self._output_tensor
 
 
 
@@ -423,7 +417,14 @@ class Layer(nn.Module):
 
         if self._built == False :
             self._input_shape =value
-            self.input_filters = -1 if self._input_shape.ndim==0 else int(self._input_shape[0])
+            if self._input_shape.ndim == 0:
+                self.input_filters = -1
+            elif len(self._input_shape) == 1:
+                self.input_filters = self._input_shape.item()
+            else:
+                self.input_filters =int(self._input_shape[0])
+
+
             self.build(self._input_shape)
             self._built = True
 
@@ -455,6 +456,28 @@ class Layer(nn.Module):
             self._output_shape = value
         self._output_shape = value
 
+    @property
+    def input(self):
+        return  NotImplemented
+
+    @property
+    def output(self):
+        """Retrieves the output tensor(s) of a layer.
+        for memory saving issue, we don'tb prefer to keep every input/output
+        tensor in every layer.You should set self.keep.output flag to True, and then
+        retrive the output tensor when the calll() is executing.
+        # Returns
+            Output tensor or list of output tensors.
+        # Raises
+            AttributeError: if the layer is connected to
+            more than one incoming layers.
+        """
+        if self.keep_output==False:
+            raise ValueError('Layer {0} has not set self.keep.output  to True, cannot access output '.format(self.name))
+        return list(self._output_tensor) if isinstance(self._output_tensor,tuple) else self._output_tensor
+
+
+
     def reset_parameters(self):
         pass
 
@@ -481,24 +504,27 @@ class Layer(nn.Module):
 
     def __call__(self, *input, **kwargs):
         for hook in self._forward_pre_hooks.values():
-            result = hook(self, input)
-            if result is not None:
-                if not isinstance(result, tuple):
-                    result = (result,)
-                input = result
+            input = hook(self, input)
         if self._built==False :
-            self.input_shape =input[0].size()[1:] if isinstance(input[0],torch.Tensor) else torch.tensor(-1)
+            inp= unpack_singleton(input)
+            if isinstance(inp, (tuple, list)):
+                self.input_shape=inp[0].size()[1:]
+            elif isinstance(inp,torch.Tensor):
+                self.input_shape = inp.size()[1:]
+            else:
+                print('input shou be tensor or tuple of tensor')
+                self.input_shape=torch.tensor(-1)
             self.build(self.input_shape)
         if torch._C._get_tracing_state():
             result = self._slow_forward(*input, **kwargs)
         else:
             result = self.forward(*input, **kwargs)
             result=unpack_singleton(result)
+            if hasattr(self,'keep_output') and self.keep_output==True:
+                self._output_tensor=result
             if isinstance(result,torch.Tensor) :
                 if self._output_shape is None:
                     self.output_shape=result.size()[1:]
-                #self._output_tensor = result.clone().cpu().detach_()
-
 
 
         for hook in self._forward_hooks.values():
@@ -593,9 +619,11 @@ class Sequential(Layer):
     def __init__(self, *args,name=''):
         super(Sequential, self).__init__(name=name)
         self._built = False
+        self._output_repository=OrderedDict()
 
         if len(args) == 1 and isinstance(args[0], OrderedDict):
             for key, module in args[0].items():
+                module.name=key
                 self.add_module(key, module)
         elif len(args) == 1 and isinstance(args[0], (list,nn.ModuleList)):
             for  idx, module in enumerate(args[0]):
@@ -620,28 +648,23 @@ class Sequential(Layer):
                 accessed from this module using the given name
             module (Module): child module to be added to the module.
         """
-        final_output=None
-        if len(self._modules)>0:
-            last_module=list(self._modules.values())[-1]
-            if last_module.built and last_module.output_shape is not None:
-                final_output=to_list(last_module.output_shape)
 
-        if not isinstance(module,Layer) and module is not None:
-            raise TypeError("{} is not a Module subclass".format(
-                torch.typename(module)))
+        if not isinstance(module, Layer) and module is not None:
+            raise TypeError("{} is not a Module subclass".format(torch.typename(module)))
         elif not isinstance(name, torch._six.string_classes):
-            raise TypeError("module name should be a string. Got {}".format(
-                torch.typename(name)))
+            raise TypeError("module name should be a string. Got {}".format(torch.typename(name)))
         elif hasattr(self, name) and name not in self._modules:
             raise KeyError("attribute '{}' already exists".format(name))
         elif '.' in name:
             raise KeyError("module name can't contain \".\"")
         elif name == '':
             raise KeyError("module name can't be empty string \"\"")
-        if final_output is not None:
-            module.input_shape=final_output
+        if len(self._modules) > 0 and self._modules[list(self._modules.keys())[0]]._input_shape is not None and \
+                self._modules[list(self._modules.keys())[-1]].built and self._modules[
+            list(self._modules.keys())[-1]]._output_shape is not None:
+            module.input_shape = self._modules[list(self._modules.keys())[-1]]._output_shape
         self._modules[name] = module
-
+        self._output_shape = module._output_shape
 
     def sync_build(self):
         input_shape=None
@@ -692,7 +715,11 @@ class Sequential(Layer):
     def forward(self, *x):
         x = enforce_singleton(x)
         for module in self._modules.values():
+            if 'ShortCut2d' in module.__class__.__name__ and hasattr(module,'output_idx') and module.output_idx is not None:
+                module.skip_tensor=self._output_repository.value_list[module.output_idx]
             x = module(x)
+            if hasattr(module, 'keep_output') and module.keep_output == True:
+                self._output_repository[module.uuid] = x
         return x
 
 
@@ -951,7 +978,7 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
 
 
     print("--------------------------------------------------------------------------------------------------------------------------------")
-    line_new = "{0:^45s} {1:^20s}  {2:^20s} {3:^8s}  {4:^8s}  {5:^12s}".format("Layer (type)", "Output Shape","Weight ","Bias", "Param #", "FLOPS #")
+    line_new = "{0:^40s} {1:^20s}  {2:^20s} {3:^8s}  {4:^8s}  {5:^12s}".format("Layer (type)", "Output Shape","Weight ","Bias", "Param #", "FLOPS #")
     print(line_new)
     print("==============================================================================")
     total_params = 0
@@ -961,7 +988,7 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
     macc=0
     for layer in summary:
         # input_shape, output_shape, trainable, nb_params
-        line_new = "{0:<45s} {1:<20s}  {2:<20s} {3:<8s}  {4:<8}  {5:<12}".format(
+        line_new = "{0:<40s} {1:<20s}  {2:<20s} {3:<8s}  {4:<8}  {5:<12}".format(
             layer,
             str(summary[layer]["output_shape"]),
             str(summary[layer]["weight"] if 'weight' in summary[layer] else ''),
@@ -1325,50 +1352,55 @@ def check_keys(model, pretrained_state_dict):
     return True
 
 
-def get_signature(fn):
+def get_signature(fn,skip_default=False):
     signature = OrderedDict()
     parameters = list(inspect.signature(fn).parameters.values())
     for k in parameters:
-        signature[k.name]= k.default if k.default !=inspect._empty else None
+        if skip_default==False or (skip_default==True and k.default ==inspect._empty):
+            signature[k.name]= k.default if k.default !=inspect._empty else None
     return signature
 
 
-def try_map_args_and_call(fn, data: OrderedDict,data_feed=None):
-    arg_map = OrderedDict()
-    if isinstance(fn,Layer) :
-        for arg in fn.signature.key_list:
-            if arg in data_feed:
-                arg_map[arg]=data[data_feed[arg]]
-            else:
-                raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
-        #print('arg_map',arg_map.key_list)
-        out=fn(*arg_map.value_list)
-        return out
-    elif hasattr(fn,'signature') and callable(fn):
-        for arg in fn.signature.key_list:
-            if arg in data:
-                arg_map[arg]=data[arg]
-            elif arg in data_feed:
-                arg_map[arg]=data[data_feed[arg]]
-            else:
-                raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
-        #print('arg_map', arg_map.key_list)
-        out=fn(*arg_map.value_list)
-        return out
-    elif  callable(fn):
-
-        args=get_signature(fn).key_list
-        for arg in args:
-            if arg in  data_feed:
-                arg_map[arg]=data[data_feed[arg]]
-            else:
-                arg_map[arg] = ''
-        #print('arg_map', arg_map.key_list)
-        out = fn(*arg_map.value_list)
-        return out
+def try_map_args_and_call(fn, data: OrderedDict,data_feed=None,):
+    if isinstance(fn,torch.Tensor):
+        return fn
     else:
+        arg_map = OrderedDict()
+        if isinstance(fn,Layer) :
+            for arg in fn.signature.key_list:
+                if arg in data_feed:
+                    arg_map[arg]=data[data_feed[arg]]
+                else:
+                    raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
+            #print('arg_map',arg_map.key_list)
+            out=fn(*arg_map.value_list)
+            return out
+        elif hasattr(fn,'signature') and callable(fn):
+            for arg in fn.signature.key_list:
+                if arg in data:
+                    arg_map[arg]=data[arg]
+                elif arg in data_feed:
+                    arg_map[arg]=data[data_feed[arg]]
+                else:
 
-        print('uncomplete arg_map', arg_map.key_list)
+                    raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
+            #print('arg_map', arg_map.key_list)
+            out=fn(*arg_map.value_list)
+            return out
+        elif  callable(fn):
+
+            args=get_signature(fn).key_list
+            for arg in args:
+                if arg in  data_feed:
+                    arg_map[arg]=data[data_feed[arg]]
+                else:
+                    arg_map[arg] = ''
+            #print('arg_map', arg_map.key_list)
+            out = fn(*arg_map.value_list)
+            return out
+        else:
+
+            print('uncomplete arg_map', arg_map.key_list)
 
 
 
