@@ -10,7 +10,7 @@ from functools import partial
 import numpy as np
 from .common import *
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
-__all__ = ['reshape','argmax','log_sum_exp','reduce_max','reduce_min','reduce_mean','reduce_sum','expand_dims','shuffle','random_choice','meshgrid','element_cosine_distance','gram_matrix','get_rotation_matrix2d','warp_affine','binary_crossentropy']
+__all__ = ['reshape','argmax','gram_matrix','log_sum_exp','reduce_max','reduce_min','reduce_mean','reduce_sum','expand_dims','shuffle','random_choice','meshgrid','element_cosine_distance','gram_matrix','get_rotation_matrix2d','warp_affine','binary_crossentropy']
 
 
 
@@ -29,7 +29,18 @@ def reshape(x,shape=None)-> torch.Tensor:
     else:
         return x
 
+def gram_matrix(input):
+    a, b, c, d = input.size()  # a=batch size(=1)
+    # b=number of feature maps
+    # (c,d)=dimensions of a f. map (N=c*d)
 
+    features = input.view(a * b, c * d)  # resise F_XL into \hat F_XL
+
+    G = torch.mm(features, features.t())  # compute the gram product
+
+    # we 'normalize' the values of the gram matrix
+    # by dividing by the number of element in each feature maps.
+    return G.div(a * b * c * d)
 
 
 
