@@ -177,7 +177,7 @@ class GanCallback(GanCallbacksBase):
         except:
             PrintException()
 
-    def post_loss_calculation(self, training_context):
+    def on_loss_calculation_end(self, training_context):
         is_collect_data = training_context['is_collect_data']
 
         true_label = to_tensor(np.ones((self.D_real.size()), dtype=np.float32))
@@ -640,7 +640,7 @@ class CycleGanCallback(GanCallbacksBase):
         except:
             PrintException()
 
-    def post_loss_calculation(self, training_context):
+    def on_loss_calculation_end(self, training_context):
         model = training_context['current_model']
         current_mode = None
         is_collect_data = training_context['is_collect_data']
