@@ -37,7 +37,7 @@ class MixupCallback(RegularizationCallbacksBase):
         self.alpha=alpha
         self.loss_criterion=loss_criterion()
         self.loss_weight=loss_weight
-    def post_loss_calculation(self, training_context):
+    def on_loss_calculation_end(self, training_context):
         '''Returns mixed inputs, pairs of targets, and lambda'''
         x=to_numpy(training_context['current_input'])
         y=to_numpy(training_context['current_target'])
@@ -96,7 +96,7 @@ class CutMixCallback(RegularizationCallbacksBase):
 
         return bbx1, bby1, bbx2, bby2
 
-    def post_loss_calculation(self, training_context):
+    def on_loss_calculation_end(self, training_context):
         '''Returns mixed inputs, pairs of targets, and lambda'''
         x = to_numpy(training_context['current_input'])
         y = to_numpy(training_context['current_target'])
