@@ -22,8 +22,22 @@ elif get_backend()=='tensorflow':
 elif get_backend()=='cntk':
     from ..backend.cntk_backend import  to_numpy,to_tensor
 
+__all__ = ['nms','xywh_to_xyxy','xyxy_to_xywh']
+
+
 
 def xywh_to_xyxy(xywh):
+    '''
+
+    Parameters
+    ----------
+    xywh (centerx,centery,width, height)
+
+    Returns
+    -------
+    xyxy (x1,y1,x2,y2)
+
+    '''
     """Convert [x1 y1 w h] box format to [x1 y1 x2 y2] format."""
     if isinstance(xywh, (list, tuple)):
         # Single box given as a list of coordinates
@@ -140,7 +154,7 @@ def nms(boxes, threshold):
                 out = out.unsqueeze(0)
                 if len(out) == 0:
                     return None
-            return out
+            return out,pick
         except Exception as e:
             print(e)
             print('box_len', box_len)
