@@ -415,7 +415,7 @@ def load_lfw(kind='train', is_flatten=None, is_onehot=None):
     return dataset
 
 
-def load_examples_data(dataset_name):
+def load_examples_data(dataset_name,include_test=False):
     dataset_name = dataset_name.strip().lower()
     if dataset_name.lower() not in ['pokemon', 'hanzi', 'animals', 'nsfw', 'simpsons', 'horse2zebra', 'people',
                                     'autodrive']:
@@ -445,10 +445,11 @@ def load_examples_data(dataset_name):
         extract_archive(train_imgs_path, dirname, archive_format='tar')
         dataset = load_folder_images(dataset_name, os.path.join(dirname, 'train'), folder_as_label=True,
                                      expect_data_type=ExpectDataType.gray)
-        dataset_test = load_folder_images(dataset_name, os.path.join(dirname, 'test'), folder_as_label=True,
-                                          expect_data_type=ExpectDataType.gray)
+        if include_test:
+            dataset_test = load_folder_images(dataset_name, os.path.join(dirname, 'test'), folder_as_label=True,
+                                              expect_data_type=ExpectDataType.gray)
 
-        dataset.testdata = dataset_test.traindata
+            dataset.testdata = dataset_test.traindata
         dataset.class_names['zh-cn'] = dataset.class_names['en-us']
         return dataset
 
@@ -524,7 +525,7 @@ def load_examples_data(dataset_name):
         # print('trimap',len(masks))
 
         imgdata = ImageDataset(images=imgs, expect_data_type=ExpectDataType.rgb)
-        mskdata = MaskDataset(images=masks, expect_data_type=ExpectDataType.binary_mask)
+        mskdata = MaskDataset(masks=masks, expect_data_type=ExpectDataType.binary_mask)
         dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=mskdata))
         print('get people images :{0}'.format(len(dataset)))
         return dataset
@@ -533,8 +534,23 @@ def load_examples_data(dataset_name):
         train_imgs_path = os.path.join(dirname, 'autodrive.tar')
         extract_path = os.path.join(dirname, 'autodrive')
         extract_archive(train_imgs_path, dirname, archive_format='tar')
-        dataset = load_folder_images(dataset_name, os.path.join(dirname, 'images'), folder_as_label=False)
+        imgs = glob.glob(os.path.join(dirname, 'images', '*.*g'))
+        masks = glob.glob(os.path.join(dirname, 'masks', '*.png'))
 
+
+
+        imgdata = ImageDataset(images=imgs, expect_data_type=ExpectDataType.rgb)
+        mskdata = MaskDataset(masks=masks, expect_data_type=ExpectDataType.color_mask)
+        def parse_code(l):
+            if len(l.strip().split("\t")) == 2:
+                a, b = l.replace('\t\t', '\t').strip().split("\t")
+                return tuple(int(i) for i in b.split(' ')), a
+
+        label_codes, label_names = zip(*[parse_code(l) for l in open(os.path.join(dirname,"label_colors.txt")).readlines()])
+        for i in range(len(label_codes)):
+            mskdata.palette[label_names[i]]=label_codes[i]
+
+        dataset = DataProviderV2(dataset_name=dataset_name, traindata=Iterator(data=imgdata, label=mskdata))
         print('get autodrive images :{0}'.format(len(dataset)))
         return dataset
 
