@@ -316,7 +316,7 @@ class DataProviderV2(object):
 
         self.scenario='train'
         self._class_names={}
-        self.palettes=None
+
         self._minibatch_size = minibatch_size
         self.is_flatten=bool(kwargs['is_flatten']) if 'is_flatten' in kwargs else False
         self.__default_language__='en-us'
@@ -373,6 +373,7 @@ class DataProviderV2(object):
         else:
             return []
 
+
     @property
     def minibatch_size(self):
         return self._minibatch_size
@@ -385,6 +386,15 @@ class DataProviderV2(object):
         if self.testdata is not None and hasattr(self.testdata, '_minibatch_size'):
             self.testdata.minibatch_size= self._minibatch_size
 
+    @property
+    def palette(self):
+        if self.scenario=='test' and self.testdata is not None:
+            return self.testdata.palette
+        elif self.traindata is not None:
+            return self.traindata.palette
+        else:
+            return None
+
     @property
     def image_transform_funcs(self):
         return self._image_transform_funcs
@@ -407,10 +417,10 @@ class DataProviderV2(object):
         if len(self.image_transform_funcs) == 0:
             return image_backend_adaptive(img_data)
         if isinstance(img_data, np.ndarray):
-            # if img_data.ndim>=2:
             for fc in self.image_transform_funcs:
-                if (fc.__name__.startswith('random') and random.random<0.5) or  not fc.__name__.startswith('random'):
+                if not fc.__qualname__.startswith('random_') or  'crop' in fc.__qualname__  or  'rescale' in fc.__qualname__  or  (fc.__qualname__.startswith('random_') and random.randint(0,10)%2==0):
                     img_data = fc(img_data)
+
             img_data = image_backend_adaptive(img_data)
 
             return img_data
@@ -474,9 +484,9 @@ class DataProviderV2(object):
         if len(self.image_transform_funcs) == 0:
             return image_backend_adaptive(img_data)
         if isinstance(img_data, np.ndarray):
-            # if img_data.ndim>=2:
             for fc in self.image_transform_funcs:
-                img_data = fc(img_data)
+                if fc.__qualname__.startswith('random_') and random.randint(10)%2==0:
+                    img_data = fc(img_data)
             img_data = image_backend_adaptive(img_data)
 
             return img_data
