@@ -1,7 +1,8 @@
 import os
 import itertools
 import sys
-
+import string
+import random
 from typing import List, TypeVar, Iterable, Tuple, Union
 from enum import Enum, unique
 import numpy as np
@@ -10,6 +11,8 @@ from ..backend.common import DataSpec, PrintException,OrderedDict
 from .image_common import gray_scale, image2array, mask2array, image_backend_adaptive, reverse_image_backend_adaptive, \
     unnormalize, array2image, ExpectDataType, GetImageMode
 from .label_common import label_backend_adaptive
+from .mask_common import mask_backend_adaptive,color2label
+from .bbox_common import xywh_to_xyxy,xyxy_to_xywh
 from .samplers import *
 from ..backend.load_backend import get_backend
 
@@ -99,9 +102,9 @@ class ImageDataset(Dataset):
         if len(self.image_transform_funcs) == 0:
             return image_backend_adaptive(img_data)
         if isinstance(img_data, np.ndarray):
-            # if img_data.ndim>=2:
             for fc in self.image_transform_funcs:
-                img_data = fc(img_data)
+                if not fc.__qualname__.startswith('random_') or  'crop' in fc.__qualname__  or  'rescale' in fc.__qualname__  or  (fc.__qualname__.startswith('random_') and random.randint(0,10)%2==0):
+                    img_data = fc(img_data)
             img_data = image_backend_adaptive(img_data)
 
             return img_data
@@ -134,16 +137,22 @@ class ImageDataset(Dataset):
 
 
 class MaskDataset(Dataset):
-    def __init__(self, images=None, expect_data_type: ExpectDataType = ExpectDataType.label_mask,
+    def __init__(self, masks=None,class_names=None, expect_data_type: ExpectDataType = ExpectDataType.label_mask,
                  get_image_mode: GetImageMode = GetImageMode.processed, symbol=None, name=''):
         super().__init__(symbol=symbol, expect_data_type=expect_data_type, name=name)
-        if expect_data_type not in [ExpectDataType.label_mask, ExpectDataType.binary_mask]:
+        if expect_data_type not in [ExpectDataType.label_mask, ExpectDataType.binary_mask,ExpectDataType.alpha_mask,ExpectDataType.color_mask]:
             raise ValueError('Only mask is valid expect image type. ')
 
-        self.__add__(images)
+        self.__add__(masks)
         self.get_image_mode = get_image_mode
-        self.image_transform_funcs = []
+        self.palette=OrderedDict()
+        self.mask_transform_funcs = []
         self.is_pair_process = False
+        self.class_names = {}
+        self._lab2idx = {}
+        self._idx2lab = {}
+        if class_names is not None:
+            self.class_names = class_names
 
     def __getitem__(self, index: int):
         img = super().__getitem__(index)  # self.pop(index)
@@ -153,11 +162,31 @@ class MaskDataset(Dataset):
             return None
 
         if isinstance(img, str):
+
             if self.expect_data_type == ExpectDataType.binary_mask:
-                img = mask2array(img)
-                img[img > 0] = 255
+                img = mask2array(img).astype(np.int64)
+                mv=img.max()
+                img[img== mv] = 255
+                img[img <= 0] = 0
+                img[img > 128] = 255
+            elif self.expect_data_type == ExpectDataType.alpha_mask:
+                img = mask2array(img).astype(np.float32)
             elif self.expect_data_type == ExpectDataType.label_mask:
-                img = image2array(img).astype(np.uint8)
+                if '.png' in img:
+                    img = image2array(img)
+                    img=img[:,:,1]
+                else:
+                    img = mask2array(img).astype(np.int64)
+
+            elif self.expect_data_type == ExpectDataType.color_mask:
+                img =image2array(img)
+                if img.ndim==2:
+                    pass
+                elif img.ndim==3:
+                    img=img[:,:,:3]
+                    if len(self.palette)>0:
+                        img=color2label(img,self.palette).astype(np.int64)
+
 
         if self.get_image_mode == GetImageMode.raw:
             return img
@@ -167,20 +196,25 @@ class MaskDataset(Dataset):
             raise ValueError('image data dimension  should be 2 or 3, but get {0}'.format(img.ndim))
 
         if self.get_image_mode == GetImageMode.expect and self.is_pair_process == False:
-            return label_backend_adaptive(img)
+            return mask_backend_adaptive(img)
         elif self.get_image_mode == GetImageMode.processed and self.is_pair_process == False:
-            return self.image_transform(img)
+            return self.mask_transform(img)
         elif self.is_pair_process == True:
             return img
 
         return None
 
-    def mask_transform(self, img_data):
-        if self.expect_data_type==ExpectDataType.binary_mask:
-            img_data[img_data>0]=1
-            return img_data.astype(np.int64)
+    def mask_transform(self, mask_data):
+        if len(self.mask_transform_funcs) == 0:
+            return mask_backend_adaptive(mask_data,label_mapping=self.class_names,expect_data_type=self.expect_data_type)
+        if isinstance(mask_data, np.ndarray):
+            for fc in self.mask_transform_funcs:
+                if not fc.__qualname__.startswith('random_') or  'crop' in fc.__qualname__  or  'rescale' in fc.__qualname__  or  (fc.__qualname__.startswith('random_') and random.randint(0,10)%2==0):
+                    mask_data = fc(mask_data)
+            #mask_data = mask_backend_adaptive(mask_data)
+            return mask_data
         else:
-            return img_data
+            return mask_data
 
     @property
     def reverse_image_transform_funcs(self):
@@ -336,23 +370,13 @@ class SsdBboxDataset(BboxDataset):
 
 
     def convert_boxes_to_locations(self, center_form_boxes, center_form_priors):
-        if center_form_boxes.ndim==1:
-            center_form_boxes=np.expand_dims(center_form_boxes, 0)
-        if center_form_priors.ndim==1:
-            center_form_priors=np.expand_dims(center_form_priors, 0)
-        return np.concatenate([(center_form_boxes[..., :2] - center_form_priors[..., :2]) / center_form_priors[...,
-                                                                                            2:] / self.center_variance,
-                               np.log(center_form_boxes[..., 2:] / center_form_priors[..., 2:]) / self.size_variance],
+        if len(center_form_priors.shape) + 1 == len(center_form_boxes.shape):
+            center_form_priors = np.expand_dims(center_form_priors, 0)
+        return np.concatenate(
+            [(center_form_boxes[..., :2] - center_form_priors[..., :2]) / center_form_priors[..., 2:] / self.center_variance,
+             np.log(center_form_boxes[..., 2:] / center_form_priors[..., 2:]) / self.size_variance],
             axis=len(center_form_boxes.shape) - 1)
 
-    def center_form_to_corner_form(self, locations):
-        return np.concatenate(
-            [locations[..., :2] - locations[..., 2:] / 2, locations[..., :2] + locations[..., 2:] / 2],
-            len(locations.shape) - 1)
-
-    def corner_form_to_center_form(self, boxes):
-        return np.concatenate([(boxes[..., :2] + boxes[..., 2:]) / 2, boxes[..., 2:] - boxes[..., :2]],
-                              len(boxes.shape) - 1)
 
     def assign_priors(self, gt_boxes, gt_labels, center_form_priors, iou_threshold):
         # """Assign ground truth boxes and targets to priors.
@@ -383,36 +407,22 @@ class SsdBboxDataset(BboxDataset):
         # labels[best_target_per_prior < iou_threshold] = 0  # the backgournd id
         # boxes = gt_boxes[best_target_per_prior_index]
         # return boxes, labels
-        corner_form_priors = self.center_form_to_corner_form(center_form_priors)
-        center_form_boxs = self.corner_form_to_center_form(gt_boxes)
-
-        boxes = np.zeros((corner_form_priors.shape[0], 4)).astype(np.float32)
-        labels = np.zeros((corner_form_priors.shape[0])).astype(np.int64)
-
-        #
-        # _pred[:, 2] = _pred[:, 2] + _pred[:, 0]
-        # _pred[:, 3] = _pred[:, 3] + _pred[:, 1]
-        # _gt[:, 2] = _gt[:, 2] + _gt[:, 0]
-        # _gt[:, 3] = _gt[:, 3] + _gt[:, 1]
-        overlaps = self.iou_of(np.expand_dims(corner_form_priors, 0), np.expand_dims(gt_boxes, 1))
+        corner_form_priors = xywh_to_xyxy(center_form_priors)
+        ious = self.iou_of(np.expand_dims(gt_boxes,0), np.expand_dims(corner_form_priors,1))
         # size: num_priors
-        best_target_to_prior, best_target_to_prior_index =np.max(overlaps,axis=0), np.argmax(overlaps,axis=0)
+        best_target_per_prior, best_target_per_prior_index =np.max(ious,axis=1),np.argmax(ious,axis=1)
         # size: num_targets
-        best_prior_to_target, best_prior_to_target_index =np.max(overlaps,axis=1),np.argmax(overlaps,axis=1)
-        center_priors=center_form_priors[best_prior_to_target_index]
-
-        for target_index, prior_index in enumerate(best_prior_to_target_index):
-            gt_overlap=overlaps[target_index]
-            overlap_match=np.where(gt_overlap>(best_prior_to_target[target_index]*self.gt_overlap_tolerance))[0]
-            labels[overlap_match] = gt_labels[target_index]
-            loc= self.convert_boxes_to_locations(self.corner_form_to_center_form(gt_boxes[target_index]), center_form_priors[overlap_match])
-            boxes[overlap_match] =loc
-
-        for target_index, prior_index in enumerate(best_prior_to_target_index):
-            best_target_to_prior_index[prior_index] = target_index
-            labels[prior_index]=gt_labels[target_index]
-            loc=self.convert_boxes_to_locations(self.corner_form_to_center_form(gt_boxes[target_index]), center_form_priors[prior_index])
-            boxes[prior_index] =loc
+        best_prior_per_target, best_prior_per_target_index =np.max(ious,axis=0), np.argmax(ious,axis=0)
+        for target_index, prior_index in enumerate(best_prior_per_target_index):
+            best_target_per_prior_index[prior_index] = target_index
+        # 2.0 is used to make sure every target has a prior assigned
+        best_prior_per_target_index_list=best_prior_per_target_index.tolist()
+        for i in range(best_target_per_prior.shape[0]):
+            if i in best_prior_per_target_index_list:
+                best_target_per_prior[i]=2
+        labels = gt_labels[best_target_per_prior_index]
+        labels[best_target_per_prior < iou_threshold] = 0  # the backgournd id
+        boxes = gt_boxes[best_target_per_prior_index]
         return boxes, labels
 
     def binding_class_names(self, class_names=None, language=None):
@@ -428,19 +438,18 @@ class SsdBboxDataset(BboxDataset):
         if bbox is None or len(bbox) == 0:
             return np.zeros((self.priors.shape[0], 4)).astype(np.float32), np.zeros((self.priors.shape[0])).astype(
                 np.int64)
-        else:
+        elif isinstance(bbox,np.ndarray):
             height, width = self.image_size
             bbox[:, 0] = bbox[:, 0]/width
             bbox[:, 2] = bbox[:, 2]/width
             bbox[:, 1] = bbox[:, 1]/height
             bbox[:, 3] = bbox[:, 3] /height
-            if bbox is None or len(bbox) == 0:
-                boxes, labels = self.assign_priors(None, None, center_form_to_corner_form(to_numpy(self.priors)), 0.3)
-                return to_numpy(boxes), to_numpy(labels)
-            elif bbox.shape[-1] == 5:
+            if bbox.shape[-1] == 5:
                 gt_box = bbox[:, :4]
                 gt_label = bbox[:, 4]
                 boxes, labels = self.assign_priors(gt_box, gt_label, to_numpy(self.priors), 0.3)
+                boxes = xyxy_to_xywh(boxes)
+                locations = self.convert_boxes_to_locations(boxes, self.priors)
                 return boxes.astype(np.float32), labels.astype(np.int64)
             else:
                 return bbox
@@ -502,6 +511,8 @@ class Iterator(object):
         self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
 
+
+
     @property
     def data(self):
         return self._data
@@ -544,6 +555,13 @@ class Iterator(object):
         self.batch_sampler = BatchSampler(self, self._minibatch_size, is_shuffle=True, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
 
+    @property
+    def palette(self):
+        if isinstance(self._label,MaskDataset) and self._label.expect_data_type in [ExpectDataType.label_mask,ExpectDataType.color_mask]:
+            return self._label.palette
+        else:
+            return None
+
     @property
     def minibatch_size(self):
         return self._minibatch_size
@@ -631,14 +649,19 @@ class Iterator(object):
                     else:
                         mask = new_label
                 else:
-                    mask = label
+                    mask = label.copy()
                 label = None
+            else:
+                if hasattr(self.label, 'label_transform'):
+                    label = self.label.label_transform(label)
 
 
-            if hasattr(self.label, 'label_transform') and not isinstance(self.label, BboxDataset):
+
+            if hasattr(self.label, 'label_transform') and not isinstance(self.label, (BboxDataset,MaskDataset)):
                 label = self.label.label_transform(label)
 
             unpair = self.unpair.__getitem__(index % len(self.unpair)) if len(self.unpair) > 0 else None
+
             return_data = []
             if self.signature is None or len(self.signature)==0:
                 self.signature=OrderedDict()
