@@ -18,7 +18,8 @@ from itertools import repeat
 import cv2
 
 __all__ = ['read_image','read_mask','save_image','save_mask','image2array','array2image','mask2array','array2mask','list_pictures','normalize','unnormalize','channel_reverse','blur','random_blur','random_crop','resize','rescale','add_noise','gray_scale','to_rgb','auto_level','random_invert_color',
-           'image_backend_adaptive','reverse_image_backend_adaptive','random_adjust_hue', 'random_channel_shift', 'random_cutout','random_rescale_crop','random_center_crop','adjust_gamma','random_adjust_gamma','adjust_contast','random_adjust_contast','clahe','erosion_then_dilation','image_erosion','image_dilation','adaptive_binarization','random_transform']
+           'image_backend_adaptive','reverse_image_backend_adaptive','random_adjust_hue', 'random_channel_shift', 'random_cutout','random_rescale_crop','random_center_crop','adjust_gamma','random_adjust_gamma','adjust_contast','random_adjust_contast','clahe','erosion_then_dilation','image_erosion','image_dilation','adaptive_binarization','random_transform',
+           'horizontal_flip','random_mirror']
 
 
 _session=get_session()
@@ -165,6 +166,9 @@ def resize(size,keep_aspect=True,order=1,align_corner=True):
             if align_corner==False:
                 pady = ((size[0] - int(np.round(results[0].shape[0]*scale))) // 2,(size[0] - int(np.round(results[0].shape[0]*scale))) - (size[0] - int(np.round(results[0].shape[0]*scale))) // 2)
                 padx = ((size[1] - int(np.round(results[0].shape[1]*scale))) // 2,(size[1] - int(np.round(results[0].shape[1]*scale))) - (size[1] - int(np.round(results[0].shape[1]*scale))) // 2)
+            else:
+                pady = (0,(size[0] - int(np.round(results[0].shape[0] * scale))) )
+                padx = (0, (size[1] - int(np.round(results[0].shape[1] * scale))))
             all_pad = (padx, pady)
             for i in range(len(results)):
                 im=results[i]
@@ -178,7 +182,15 @@ def resize(size,keep_aspect=True,order=1,align_corner=True):
                     im[:, 2] += padx[0]
                     im[:, 3] += pady[0]
                     results[i] = im
+                elif im.ndim == 2 and im.dtype==np.int64:  # bbox
+                    im = transform.rescale(im, scale, clip=False, anti_aliasing=False,
+                                           multichannel= False,preserve_range=True,
+                                           order=0).astype(np.int64)
+                    all_pad = (pady, padx)
+                    results[i] = np.pad(im, all_pad, 'constant').astype(np.int64)
+
                 else:
+                    im=im.astype(np.float32)
                     im = transform.rescale(im, scale, clip=False, anti_aliasing=True, multichannel=True if len(im.shape) == 3 else False,order=0 if im.ndim== 2 else order)
                     if im.ndim== 2:
                         all_pad = (pady,padx)
@@ -205,11 +217,11 @@ def resize(size,keep_aspect=True,order=1,align_corner=True):
                     im[:, 3] *=img_op.scaley
                     results[i] = im
                 else:
+                    im = im.astype(np.float32)
                     results[i]=transform.resize(im, size,anti_aliasing=True,order=0 if im.ndim== 2 else order)
             return unpack_singleton(tuple(results))
     return img_op
 
-
 def rescale(scale,order=1):
     def img_op(*image:np.ndarray):
         results = to_list(image)
@@ -225,14 +237,18 @@ def rescale(scale,order=1):
                 im[:, 2] *= scale
                 im[:, 3] *= scale
                 results[i] = im
+            elif im.ndim == 2 and im.dtype == np.int64:  # bbox
+                results[i] = transform.rescale(im, (scale,scale),  clip=True, anti_aliasing=False,
+                                       multichannel=False, preserve_range=True,
+                                       order=0).astype(np.int64)
+
             else:
+                im = im.astype(np.float32)
                 results[i] = transform.rescale(im, (scale,scale), clip=False, anti_aliasing=True, multichannel=True if len(im.shape) == 3 else False,order=0 if im.ndim== 2 else order)
         return unpack_singleton(tuple(results))
     img_op.scale=(scale,scale)
     return img_op
 
-
-
 def invert_color():
     def img_op(image:np.ndarray):
         if np.min(image)>=0 and np.max(image)<=1:
@@ -262,9 +278,7 @@ def random_invert_color():
 def gray_scale():
     def img_op(image:np.ndarray):
         if image.shape[0]==3:
-            image=color.rgb2gray(image/255.0).astype(np.float32)
-            if image.max()<=1:
-                image=image*255.0
+            image=color.rgb2gray(image.astype(np.float32))
         return image
     return img_op
 
@@ -317,7 +331,7 @@ def random_adjust_contast(scale=(0.5, 1.5)):
     return img_op
 
 
-def random_adjust_hue(hue_range=(-20,20),saturation_range=(-50,50),lightness_range=(-30,30)):
+def random_adjust_hue(hue_range=(-20,20),saturation_range=(-10,10),lightness_range=(-20,20)):
     def img_op(image: np.ndarray):
         # hue is mapped to [0, 1] from [0, 360]
         # if hue_offset not in range(-180, 180):
@@ -329,11 +343,15 @@ def random_adjust_hue(hue_range=(-20,20),saturation_range=(-50,50),lightness_ran
         hue_offset=random.uniform(*hue_range)
         saturation=random.uniform(*saturation_range)
         lightness = random.uniform(*lightness_range)
-        image = color.rgb2hsv(image.astype('uint8'))  #.astype('uint8')
+        image = color.rgb2hsv(image.astype('uint8'))
         hue_offset = ((180 + hue_offset) % 180) / 360.0
         image[:, :, 0] = image[:, :, 0] + hue_offset
-        image[:, :, 1] = image[:, :, 1] + saturation / 200.0
-        image[:, :, 2] = image[:, :, 2] + lightness / 200.0
+        image[:, :, 0][image[:, :, 0] > 1] -=1
+        image[:, :, 0][image[:, :, 0] < 0.0] += 1
+
+
+        # image[:, :, 1] = image[:, :, 1] + saturation / 200.0
+        # image[:, :, 2] = image[:, :, 2] + lightness / 200.0
         image = color.hsv2rgb(image) * 255.0
         return image.astype(np.float32)
     return img_op
@@ -402,10 +420,12 @@ def random_crop(h, w):
     return img_op
 
 
-def random_rescale_crop(h,w,order=1):
+
+def random_rescale_crop(h,w,scale=(0.5,2),order=1):
+    scalemin, scalemax = scale
     def img_op(*image:np.ndarray):
         results = to_list(image)
-        scale=np.random.choice(np.clip(np.random.normal(1,0.3,100),0.5,2))
+        scale=np.random.choice(np.random.uniform(scalemin,scalemax,100))
         height, width = image[0].shape[:2]
         height, width = int(height * scale), int(width * scale)
         offset_x=0
@@ -443,10 +463,10 @@ def random_rescale_crop(h,w,order=1):
                     results[i] = None
 
             elif im.ndim==2:
-                im = transform.rescale(im, (scale, scale), clip=False, anti_aliasing=True,
-                                       multichannel=True if len(im.shape) == 3 else False,
-                                       order=0 if im.ndim == 2 else order)
-                returnData= np.zeros((h, w), dtype=np.float32)
+
+                im = np.round(transform.rescale(im, (scale, scale), clip=False, anti_aliasing=False,
+                                       multichannel= False, preserve_range=True, order=0)).astype(im.dtype)
+                returnData= np.zeros((h, w), dtype=im.dtype)
                 crop_im = im[offset_y:offset_y + h, offset_x:offset_x + w]
                 returnData[offset_y1:offset_y1+crop_im.shape[0], offset_x1:offset_x1+crop_im.shape[1]] = crop_im
                 results[i] = returnData
@@ -490,9 +510,9 @@ def random_center_crop(w, h, scale=(0.8, 1.2)):
                 area=(im[:, 3]- im[:, 1])* (im[:, 2]- im[:, 0])
                 im= im[area>0]
                 if len(im)>0:
-                    results[i]=im
+                    results[k]=im
                 else:
-                    results[i] = None
+                    results[k] = None
 
             elif im.ndim==3:
                 blank = np.zeros((max_value, max_value, im.shape[-1]))
@@ -504,17 +524,65 @@ def random_center_crop(w, h, scale=(0.8, 1.2)):
                 returnData[i1:i1 + resized_im.shape[0], j1:j1 + resized_im.shape[1], :] = resized_im
                 results[k]=returnData[i2:i2 + h, j2:j2 + w, :]
             elif im.ndim==2:
-                blank = np.zeros((max_value, max_value))
+                blank = np.zeros((max_value, max_value)).astype(im.dtype)
                 blank[i:i+height,j:j+width]=im
 
-                resized_im = transform.resize(blank, (resized_h,resized_w),anti_aliasing=True,clip=False, order=0 if blank.ndim== 2 else 1)
-                returnData= np.zeros((max(resized_h,h), max(resized_w,w)))
+                resized_im = np.round(transform.resize(blank, (resized_h,resized_w),anti_aliasing=False,clip=False, preserve_range=True,order=0)).astype(im.dtype)
+                returnData= np.zeros((max(resized_h,h), max(resized_w,w))).astype(im.dtype)
 
                 returnData[i1:i1 + resized_im.shape[0], j1:j1 + resized_im.shape[1]] = resized_im
                 results[k]=returnData[i2:i2 + h, j2:j2 + w]
         return unpack_singleton(tuple(results))
     return img_op
 
+def horizontal_flip():
+    # horizontal flip doesn't need skimage, it's easy as flipping the image array of pixels !
+    def img_op(*image: np.ndarray):
+        results = to_list(image)
+        height, width = image[0].shape[:2]
+        for k in range(len(results)):
+            im = results[k]
+            if im is None:
+                pass
+            elif im.ndim == 2 and im.shape[-1] in (4, 5):  # bbox
+                im[:, 0::2] = width - im[:, 2::-2]
+                if len(im) > 0:
+                    results[k] = im
+                else:
+                    results[k] = None
+            elif im.ndim == 3:
+                results[k] = im[:, ::-1]
+            elif im.ndim == 2:
+                results[k] = im[::-1]
+        return unpack_singleton(tuple(results))
+    return img_op
+
+def random_mirror():
+    def img_op(*image:np.ndarray):
+        results = to_list(image)
+        height, width = image[0].shape[:2]
+        img_op.rnd=random.randint(0,10)
+        for k in range(len(results)):
+            im = results[k]
+            if im is None:
+                pass
+            elif im.ndim == 2 and im.shape[-1] in (4, 5):  # bbox
+                if img_op.rnd % 2 == 0:
+                    im[:, 0::2] = width - im[:, 2::-2]
+                if len(im)>0:
+                    results[k]=im
+                else:
+                    results[k] = None
+
+            elif im.ndim==3:
+                if img_op.rnd%2==0:
+                    results[k]=im[:, ::-1]
+            elif im.ndim==2:
+                if img_op.rnd % 2 == 0:
+                    results[k] = im[:, ::-1]
+        return unpack_singleton(tuple(results))
+    return img_op
+
 
 def image_backend_adaptive(image):
     if _session.backend=='tensorflow' and image.ndim in (3,4):
@@ -608,24 +676,36 @@ def image_dilation(filter_size=3,repeat=1):
 
 def adaptive_binarization(threshold_type='otsu' ):
     def img_op(image: np.ndarray):
+        image=image.astype(np.float32)
         local_thresh=None
-        blur = gaussian(image, sigma=1)
-        if threshold_type=='otsu':
-            local_thresh = threshold_otsu(blur)
-        elif threshold_type=='minimum':
-            local_thresh= threshold_minimum(blur)
-        elif threshold_type=='local':
-            local_thresh=threshold_local(blur, block_size=35, offset=10)
-        elif threshold_type=='isodata':
-            local_thresh=threshold_isodata(blur, nbins=256)
-        image =(image > local_thresh).astype(np.float32)*255.0
+        blur = gaussian(image, sigma=0.2)
+        try:
+            if threshold_type=='otsu'  :
+                if len(blur.shape) > 2 and blur.shape[-1] in (3, 4):
+                    image = color.rgb2gray(blur.astype(np.float32))
+                if blur.min() == blur.max():
+                    return image
+                else:
+                    local_thresh = threshold_otsu(blur)
+            elif threshold_type=='minimum':
+                local_thresh= threshold_minimum(blur)
+            elif threshold_type=='local':
+                local_thresh=threshold_local(blur, block_size=35, offset=10)
+            elif threshold_type=='isodata':
+                local_thresh=threshold_isodata(blur, nbins=256)
+            image =(image > local_thresh).astype(np.float32)*255.0
+        except Exception as e:
+            print(e)
         return image
     return img_op
 
 
-def blur(sigma=1):
+def blur(sigma=0.3):
     def img_op(image: np.ndarray):
-        image =  gaussian(image, sigma=sigma,multichannel=True)
+        if len(image.shape) > 2 and image.shape[-1] in (3, 4):
+            image =  gaussian(image.astype(np.float32), sigma=sigma,multichannel=True,preserve_range=True)
+        else:
+            image = gaussian(image.astype(np.float32), sigma=sigma, multichannel=False,preserve_range=True)
         return image
     return img_op
 
@@ -635,7 +715,7 @@ def random_blur():
         if n==0:
             return image
         else:
-            image =  gaussian(image, sigma=n,multichannel=True)
+            image =  gaussian(image.astype(np.float32), sigma=np.random.choice(np.arange(0.1,0.6,0.05)),multichannel=True)
         return image
     return img_op
 
