@@ -17,6 +17,7 @@ from  sys import stderr
 import requests
 import warnings
 import numpy as np
+from socket import create_connection
 from tqdm import tqdm
 import gzip
 import tarfile
@@ -26,6 +27,8 @@ import hashlib
 import glob
 import six
 from scipy.io import loadmat
+from urllib3.exceptions import NewConnectionError
+
 from ..backend.common import OrderedDict,PrintException
 from six.moves.urllib.error import HTTPError
 from six.moves.urllib.error import URLError
@@ -35,7 +38,12 @@ try:
 except ImportError:
     from six.moves.urllib.request import urlretrieve
 
-
+def is_connected():
+    try:
+        urlopen('https://docs.google.com')
+        return True
+    except :
+        return False
 
 def ensure_dir(dirpath):
     if not os.path.exists(dirpath): os.makedirs(dirpath)
@@ -155,21 +163,17 @@ def download_file_from_google_drive(file_id, dirname, filename=None, md5=None):
         print('archive file is already existing, donnot need download again.')
     else:
         with TqdmProgress(unit='B', unit_scale=True, miniters=10, desc='') as t:  # all optional kwargs
-            try:
-                session = requests.Session()
-                response = session.get(url, params={'id': file_id}, stream=True)
-                token = _get_confirm_token(response)
+            session = requests.Session()
+            response = session.get(url, params={'id': file_id}, stream=True)
+            token = _get_confirm_token(response)
 
-                if token:
-                    params = {'id': file_id, 'confirm': token}
-                    response = session.get(url, params=params, stream=True)
+            if token:
+                params = {'id': file_id, 'confirm': token}
+                response = session.get(url, params=params, stream=True)
 
-                _save_response_content(response, fpath)
-                _write_h(dirname,True,False)
-                return True
-            except:
-                PrintException()
-                return False
+            _save_response_content(response, fpath)
+            _write_h(dirname,True,False)
+            return True
     return False
 
 
@@ -189,48 +193,61 @@ def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
     if not filename:
         filename = file_id
     fpath = os.path.join(dirname, filename)
-    if os.path.exists(os.path.join(dirname, 'models_md5.json')):
-        os.remove(os.path.join(dirname, 'models_md5.json'))
-
-    isload=download_file_from_google_drive('1AujFsfyzyYUcUrNI_GT1dvNrHaXIQkGz', dirname, 'models_md5.json')
-    models_md5 = OrderedDict()
-    need_download = True
-    if isload==False:
+    check_internet = is_connected()
+    if check_internet == False:
         if os.path.exists(os.path.join(dirname, filename)):
-            stderr.write('download error, use local exsiting model')
+            print('internet connect  error,model file is already existing, donnot need download again.')
+            return True
         else:
-            stderr.write('sorry, download failed, cannot get the pretrained model.')
+            raise NewConnectionError
     else:
-        need_download=True
+        isload = False
+        models_md5 = None
+        need_download = True
         if os.path.exists(os.path.join(dirname, 'models_md5.json')):
             with open(os.path.join(dirname, 'models_md5.json')) as f:
                 models_md5 = json.load(f)
-
-        # if os.path.exists(os.path.join(dirname, filename)):
-        #     print('archive file is already existing, donnot need download again.')
-
-        if os.path.exists(os.path.join(dirname, filename)) :
-            if check_integrity(os.path.join(dirname, filename), models_md5[filename]):
-                need_download=False
-                print('model file is already existing, donnot need download again.')
-            else:
-                print('Your pretrained model has newer version, will you want to update it?')
-                ans = input('(Y/N) << ').lower()
-                if ans in ['yes', 'y']:
-                    os.remove(os.path.join(dirname, filename))
+            os.remove(os.path.join(dirname, 'models_md5.json'))
+        try:
+            isload = download_file_from_google_drive('1AujFsfyzyYUcUrNI_GT1dvNrHaXIQkGz', dirname, 'models_md5.json')
+            models_md5 = OrderedDict()
+            need_download = True
+            if isload == False:
+                if os.path.exists(os.path.join(dirname, filename)):
+                    stderr.write('download error, use local exsiting model')
                 else:
+                    stderr.write('sorry, download failed, cannot get the pretrained model.')
+            else:
+                need_download = True
+                if os.path.exists(os.path.join(dirname, 'models_md5.json')):
+                    with open(os.path.join(dirname, 'models_md5.json')) as f:
+                        models_md5 = json.load(f)
+
+            if os.path.exists(os.path.join(dirname, filename)):
+                if check_integrity(os.path.join(dirname, filename), models_md5[filename]):
                     need_download = False
+                    print('model file is already existing, donnot need download again.')
+                else:
+                    print('Your pretrained model has newer version, will you want to update it?')
+                    ans = input('(Y/N) << ').lower()
+                    if ans in ['yes', 'y']:
+                        os.remove(os.path.join(dirname, filename))
+                    else:
+                        need_download = False
 
-        if need_download:
-            with TqdmProgress(unit='B', unit_scale=True, miniters=10, desc='') as t:  # all optional kwargs
-                session = requests.Session()
-                response = session.get(url, params={'id': file_id}, stream=True)
-                token = _get_confirm_token(response)
+            if need_download:
+                with TqdmProgress(unit='B', unit_scale=True, miniters=10, desc='') as t:  # all optional kwargs
+                    session = requests.Session()
+                    response = session.get(url, params={'id': file_id}, stream=True)
+                    token = _get_confirm_token(response)
+                    if token:
+                        params = {'id': file_id, 'confirm': token}
+                        response = session.get(url, params=params, stream=True)
+                        _save_response_content(response, fpath)
+        except Exception as e:
+            print(e)
+            PrintException()
 
-                if token:
-                    params = {'id': file_id, 'confirm': token}
-                    response = session.get(url, params=params, stream=True)
-                _save_response_content(response, fpath)
 
 def _get_confirm_token(response):
     for key, value in response.cookies.items():
