@@ -50,7 +50,7 @@ _quadruple = _ntuple(4)
 class Conv1d_Block(Layer):
     def __init__(self, kernel_size=3, num_filters=None, strides=1, auto_pad=True, padding_mode='zero',
                  activation=None, normalization=None, use_bias=False, dilation=1, groups=1, add_noise=False,
-                 noise_intensity=0.005, dropout_rate=0, name=None, filter_rate=None, **kwargs):
+                 noise_intensity=0.005, dropout_rate=0, name=None, depth_multiplier=None, **kwargs):
         super(Conv1d_Block, self).__init__()
         self.kernel_size = kernel_size
         self.num_filters = num_filters
@@ -67,14 +67,14 @@ class Conv1d_Block(Layer):
         self.dilation = dilation
         self.groups = groups
 
-        self.add_noise = False
+        self.add_noise = add_noise
         self.noise_intensity = noise_intensity
         self.dropout_rate = dropout_rate
         self.conv = None
         self.norm = get_normalization(normalization)
         self.activation = get_activation(activation)
-        self.droupout = None
-        self.filter_rate = filter_rate
+
+        self.depth_multiplier = depth_multiplier
         self._name = name
 
     def build(self, input_shape):
@@ -82,7 +82,7 @@ class Conv1d_Block(Layer):
             self.conv = Conv1d(kernel_size=self.kernel_size, num_filters=self.num_filters, strides=self.strides,
                                auto_pad=self.auto_pad, padding_mode=self.padding_mode, activation=None,
                                use_bias=self.use_bias, dilation=self.dilation, groups=self.groups, name=self._name,
-                               filter_rate=self.filter_rate).to(_device)
+                               depth_multiplier=self.depth_multiplier).to(_device)
             self.conv.input_shape = input_shape
 
             output_shape = self._input_shape.clone().tolist()
@@ -118,7 +118,7 @@ class Conv1d_Block(Layer):
 class Conv2d_Block(Layer):
     def __init__(self, kernel_size=(3, 3), num_filters=None, strides=1, auto_pad=True, padding_mode='zero',
                  activation=None, normalization=None, use_spectral=False, use_bias=False, dilation=1, groups=1,
-                 add_noise=False, noise_intensity=0.005, dropout_rate=0, name=None, filter_rate=None, **kwargs):
+                 add_noise=False, noise_intensity=0.005, dropout_rate=0, name=None, depth_multiplier=None, **kwargs):
         super(Conv2d_Block, self).__init__()
         self.kernel_size = kernel_size
         self.num_filters = num_filters
@@ -135,7 +135,7 @@ class Conv2d_Block(Layer):
         self.dilation = dilation
         self.groups = groups
 
-        self.add_noise = False
+        self.add_noise = add_noise
         self.noise_intensity = noise_intensity
         self.dropout_rate = dropout_rate
         self.conv = None
@@ -143,7 +143,7 @@ class Conv2d_Block(Layer):
         self.norm = get_normalization(normalization)
         self.activation = get_activation(activation)
         self.droupout = None
-        self.filter_rate = filter_rate
+        self.depth_multiplier = depth_multiplier
         self._name = name
 
     def build(self, input_shape):
@@ -151,7 +151,7 @@ class Conv2d_Block(Layer):
             conv = Conv2d(kernel_size=self.kernel_size, num_filters=self.num_filters, strides=self.strides,
                           auto_pad=self.auto_pad, padding_mode=self.padding_mode, activation=None,
                           use_bias=self.use_bias, dilation=self.dilation, groups=self.groups, name=self._name,
-                          filter_rate=self.filter_rate).to(_device)
+                          depth_multiplier=self.depth_multiplier).to(_device)
             conv.input_shape = input_shape
 
             if self.use_spectral:
@@ -173,7 +173,7 @@ class Conv2d_Block(Layer):
             x = self.norm(x)
         if self.activation is not None:
             x = self.activation(x)
-        if self.dropout_rate > 0:
+        if self.dropout_rate > 0 :
             x = F.dropout(x, p=self.dropout_rate, training=self.training)
         return x
 
@@ -190,7 +190,7 @@ class Conv2d_Block(Layer):
 class TransConv2d_Block(Layer):
     def __init__(self, kernel_size=(3, 3), num_filters=None, strides=1, auto_pad=True, padding_mode='zero',
                  activation=None, normalization=None, use_spectral=False, use_bias=False, dilation=1, groups=1,
-                 add_noise=False, noise_intensity=0.005, dropout_rate=0, name=None, filter_rate=None, **kwargs):
+                 add_noise=False, noise_intensity=0.005, dropout_rate=0, name=None, depth_multiplier=None, **kwargs):
         super(TransConv2d_Block, self).__init__()
         self.kernel_size = kernel_size
         self.num_filters = num_filters
@@ -201,7 +201,7 @@ class TransConv2d_Block(Layer):
         self.use_bias = use_bias
         self.dilation = dilation
         self.groups = groups
-        self.add_noise = False
+        self.add_noise = add_noise
         self.noise_intensity = noise_intensity
         self.dropout_rate = dropout_rate
         self.conv = None
@@ -209,14 +209,14 @@ class TransConv2d_Block(Layer):
         self.norm = get_normalization(normalization)
         self.activation = get_activation(activation)
         self.droupout = None
-        self.filter_rate = filter_rate
+        self.depth_multiplier = depth_multiplier
 
     def build(self, input_shape):
         if self._built == False or self.conv is None:
             conv = TransConv2d(kernel_size=self.kernel_size, num_filters=self.num_filters, strides=self.strides,
                                     auto_pad=self.auto_pad, padding_mode=self.padding_mode, activation=None,
                                     use_bias=self.use_bias, dilation=self.dilation, groups=self.groups, name=self.name,
-                                    filter_rate=self.filter_rate).to(_device)
+                                    depth_multiplier=self.depth_multiplier).to(_device)
             conv.input_shape = input_shape
             if self.use_spectral:
                 self.conv = nn.utils.spectral_norm(conv)
@@ -255,7 +255,7 @@ class TransConv2d_Block(Layer):
 class DepthwiseConv2d_Block(Layer):
     def __init__(self, kernel_size=(3, 3), depth_multiplier=1, strides=1, auto_pad=True, padding_mode='zero',
                  activation=None, normalization=None, use_spectral=False,use_bias=False, dilation=1, add_noise=False,
-                 noise_intensity=0.005, dropout_rate=0, name=None, filter_rate=None, **kwargs):
+                 noise_intensity=0.005, dropout_rate=0, name=None, **kwargs):
         super(DepthwiseConv2d_Block, self).__init__()
         self.kernel_size = kernel_size
         self.depth_multiplier = depth_multiplier
@@ -272,7 +272,7 @@ class DepthwiseConv2d_Block(Layer):
         self.use_bias = use_bias
         self.dilation = dilation
 
-        self.add_noise = False
+        self.add_noise = add_noise
         self.noise_intensity = noise_intensity
         self.dropout_rate = dropout_rate
         self.conv =None
@@ -323,11 +323,11 @@ class DepthwiseConv2d_Block(Layer):
 class SeparableConv2d_Block(Layer):
     def __init__(self, kernel_size=(3, 3), depth_multiplier=1, strides=1, auto_pad=True, padding_mode='zero',
                  activation=None, normalization=None, use_spectral=False, use_bias=False, dilation=1, groups=1,
-                 add_noise=False, noise_intensity=0.005, dropout_rate=0, name=None, filter_rate=None, **kwargs):
+                 add_noise=False, noise_intensity=0.005, dropout_rate=0, name='', **kwargs):
         super(SeparableConv2d_Block, self).__init__()
         self.kernel_size = kernel_size
         self.depth_multiplier = depth_multiplier
-
+        self.num_filters=kwargs.get('num_filters')
         self.strides = strides
         self.auto_pad = auto_pad
         self.padding = 0
@@ -341,23 +341,24 @@ class SeparableConv2d_Block(Layer):
         self.dilation = dilation
         self.groups = groups
 
-        self.add_noise = False
+        self.add_noise = add_noise
         self.noise_intensity = noise_intensity
         self.dropout_rate = dropout_rate
         self.conv = None
         self.use_spectral = use_spectral
         self.norm = get_normalization(normalization)
         self.activation = get_activation(activation)
-        self.droupout = None
-        self.filter_rate = filter_rate
+
+        self.depth_multiplier = depth_multiplier
         self._name = name
 
     def build(self, input_shape):
         if self._built == False:
+            self.num_filters=self.input_filters*self.depth_multiplier if self.num_filters is None else self.num_filters
             conv = DepthwiseConv2d(kernel_size=self.kernel_size, depth_multiplier=self.depth_multiplier,
                                    strides=self.strides, auto_pad=self.auto_pad, padding_mode=self.padding_mode,
                                    activation=None, use_bias=self.use_bias, dilation=self.dilation, groups=self.groups,
-                                   name=self._name, filter_rate=self.filter_rate).to(_device)
+                                   name=self._name).to(_device)
             conv.input_shape = input_shape
 
             if self.use_spectral:
@@ -413,11 +414,11 @@ class GcdConv2d_Block(Layer):
         self.dilation = dilation
         self.groups = groups
         self.weights_contraint = weights_contraint
-        self.add_noise = False
+        self.add_noise = add_noise
         self.noise_intensity = noise_intensity
         self.dropout_rate = dropout_rate
         self.conv = None
-        self.droupout = None
+
         self.divisor_rank = divisor_rank
 
         self.activation = get_activation(activation)
@@ -478,7 +479,7 @@ class GcdConv2d_Block_1(Layer):
         self.dilation = dilation
         self.groups = groups
         self.weights_contraint = weights_contraint
-        self.add_noise = False
+        self.add_noise = add_noise
         self.noise_intensity = noise_intensity
         self.dropout_rate = dropout_rate
         self.activation = get_activation(activation)
@@ -626,7 +627,7 @@ class Classifier1d(Layer):
 
 
 class ShortCut2d(Layer):
-    def __init__(self, *args, activation=None, mode=ShortcutMode.add, name='shortcut', **kwargs):
+    def __init__(self, *args, output_idx=None,activation=None, mode='add', name='shortcut', **kwargs):
         """
 
         Parameters
@@ -637,9 +638,11 @@ class ShortCut2d(Layer):
         self.activation = get_activation(activation)
         self.has_identity = False
         self.mode = mode if isinstance(mode, str) else mode.value
+        self.output_idx=output_idx
+        self.skip_tensor=None
         for i in range(len(args)):
             arg = args[i]
-            if isinstance(arg, (Layer, list, dict)):
+            if isinstance(arg, (Layer,torch.Tensor, list, dict)):
                 if isinstance(arg, list):
                     arg = Sequential(*arg)
                 elif isinstance(arg, dict) and len(args) == 1:
@@ -651,12 +654,14 @@ class ShortCut2d(Layer):
                             self.add_module(k, v)
                 elif isinstance(arg, dict) and len(args) > 1:
                     raise ValueError('more than one dict argument is not support.')
+                elif isinstance(arg, torch.Tensor):
+                    self.skip_tensor=arg
                 elif isinstance(arg, Identity):
                     self.has_identity = True
                     self.add_module('Identity', arg)
                 else:
                     self.add_module('branch{0}'.format(i + 1), arg)
-        if len(self._modules) == 1 and self.has_identity == False:
+        if len(self._modules) == 1 and self.has_identity == False and self.output_idx is None:
             self.has_identity = True
             self.add_module('Identity', Identity())
 
@@ -670,10 +675,12 @@ class ShortCut2d(Layer):
         for k, v in self._modules.items():
             if not isinstance(v, Identity):
                 result.append(v(x))
+        if hasattr(self,'skip_tensor') and self.skip_tensor is not None:
+            result.append(self.skip_tensor)
 
         if (not hasattr(self, 'mode') or self.mode == 'add'):
             if result[0].shape!=result[1].shape:
-                print('')
+                raise RuntimeError('Shape is not compatable between  {0} and {1}'.format(result[0].shape,result[1].shape))
             result = Add()(result)
         elif self.mode == 'dot':
             result = Dot()(result)
