@@ -26,10 +26,10 @@ import torchvision
 from .pytorch_normalizations import *
 import numpy as np
 
-__all__ = ['Dense', 'Flatten', 'Concatenate', 'Concate', 'Add', 'Subtract', 'Dot', 'Conv1d', 'Conv2d', 'Conv3d',
+__all__ = ['Dense', 'Flatten', 'Concatenate', 'Concate','SoftMax','Add', 'Subtract', 'Dot', 'Conv1d', 'Conv2d', 'Conv3d',
            'TransConv1d', 'TransConv2d', 'TransConv3d', 'SeparableConv1d', 'SeparableConv2d', 'SeparableConv3d',
            'DepthwiseConv1d', 'DepthwiseConv2d', 'DepthwiseConv3d', 'GcdConv2d', 'GcdConv2d_1', 'Lambda', 'Reshape',
-           'CoordConv2d', 'Upsampling2d', 'Dropout', 'AlphaDropout', 'SelfAttention']
+           'CoordConv2d', 'Upsampling2d', 'Dropout', 'AlphaDropout', 'SelfAttention','SingleImageLayer']
 
 _session = get_session()
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
@@ -187,6 +187,7 @@ class Add(Layer):
 
     def build(self, input_shape):
         if self._built == False:
+            self.output_shape = input_shape
             self._built = True
 
     def forward(self, *x) -> torch.Tensor:
@@ -208,7 +209,7 @@ class Subtract(Layer):
 
     def build(self, input_shape):
         if self._built == False:
-            self.output_shape = input_shape[0]
+            self.output_shape = input_shape
             self._built = True
 
     def forward(self, *x) -> torch.Tensor:
@@ -230,7 +231,7 @@ class Dot(Layer):
 
     def build(self, input_shape):
         if self._built == False:
-            self.output_shape = input_shape[0]
+            self.output_shape = input_shape
             self._built = True
 
     def forward(self, *x) -> torch.Tensor:
@@ -247,18 +248,24 @@ class Dot(Layer):
 class SoftMax(Layer):
     r"""Flatten layer to flatten a tensor after convolution."""
 
-    def __init__(self, axis=1):
+    def __init__(self, axis=1, add_noise=False, noise_intensity=0.005, **kwargs):
         super(SoftMax, self).__init__()
+        self.axis = kwargs.get('dim', axis)
 
     def forward(self, *x) -> torch.Tensor:
-        if not isinstance(x, (list, tuple)):
-            raise ValueError('A merge layer should be called on a list of inputs.')
-        if not isinstance(x, tuple):
-            x = list(x)
-        out = 0
-        for item in x:
-            out = torch.sub(out, item)
-        return out
+        x = enforce_singleton(x)
+        if not hasattr(self, 'add_noise'):
+            self.add_noise = False
+            self.noise_intensity = 0.005
+        if self.training:
+
+            if self.add_noise == True:
+                noise = self.noise_intensity * torch.randn_like(x, dtype=torch.float32)
+                x = x + noise
+            x = F.log_softmax(x, dim=1)
+        else:
+            x = torch.softmax(x, dim=1)
+        return x
 
 
 _gcd = gcd
@@ -271,12 +278,12 @@ class _ConvNd(Layer):
                      'groups', 'transposed']
 
     def __init__(self, kernel_size, num_filters, strides, auto_pad, padding_mode, use_bias, dilation, groups,
-                 transposed, name, filter_rate, **kwargs):
+                 transposed, name, depth_multiplier, **kwargs):
         super(_ConvNd, self).__init__(name=name)
 
         self.num_filters = None
-        if num_filters is None and filter_rate is not None:
-            self.filter_rate = filter_rate
+        if num_filters is None and depth_multiplier is not None:
+            self.depth_multiplier = depth_multiplier
         else:
             self.num_filters = int(kwargs.get('out_channels', num_filters))
         self.kernel_size = kernel_size
@@ -317,8 +324,9 @@ class _ConvNd(Layer):
 
     def build(self, input_shape):
         if self._built == False:
-            if self.num_filters is None and self.filter_rate is not None:
-                self.num_filters = int(round(self.input_filters * self.filter_rate,0))
+            self.input_filters=input_shape[0].item()
+            if self.num_filters is None and self.depth_multiplier is not None:
+                self.num_filters = int(round(self.input_filters * self.depth_multiplier,0))
             if self.input_filters % self.groups != 0:
                 raise ValueError('in_channels must be divisible by groups')
 
@@ -341,8 +349,7 @@ class _ConvNd(Layer):
 
             batch_input_shape = self._input_shape.clone().tolist()
             batch_input_shape.insert(0, 2)
-            x = torch.rand(*batch_input_shape).to(torch.device("cuda" if self.weight[0].data.is_cuda else "cpu")).to(
-                self.weight[0].data.dtype)
+            x = torch.rand(*batch_input_shape).to(torch.device("cuda" if self.weight[0].data.is_cuda else "cpu")).to(self.weight[0].data.dtype)
             out = self.forward(x)
             if isinstance(out, (list, tuple)):
                 out = unpack_singleton(out)
@@ -380,12 +387,12 @@ class _ConvNd(Layer):
 
 class Conv1d(_ConvNd):
     def __init__(self, kernel_size, num_filters=None, strides=1, auto_pad=True, padding_mode='zero', activation=None,
-                 use_bias=False, dilation=1, groups=1, name='', filter_rate=None, **kwargs):
+                 use_bias=False, dilation=1, groups=1, name='', depth_multiplier=None, **kwargs):
         kernel_size = _single(kernel_size)
         strides = _single(strides)
         dilation = _single(dilation)
         super(Conv1d, self).__init__(kernel_size, num_filters, strides, auto_pad, padding_mode, use_bias, dilation,
-                                     groups, False, name, filter_rate, **kwargs)
+                                     groups, False, name, depth_multiplier, **kwargs)
         self.activation = get_activation(activation)
         self.auto_pad = auto_pad
         self.padding = kwargs.get('padding', None)
@@ -426,13 +433,13 @@ class Conv1d(_ConvNd):
 
 class Conv2d(_ConvNd):
     def __init__(self, kernel_size, num_filters=None, strides=1, auto_pad=True, padding_mode='zero', activation=None,
-                 use_bias=False, dilation=1, groups=1, name='', filter_rate=None, **kwargs):
+                 use_bias=False, dilation=1, groups=1, name='', depth_multiplier=None, **kwargs):
         kernel_size = _pair(kernel_size)
         strides = _pair(strides)
         dilation = _pair(dilation)
 
         super(Conv2d, self).__init__(kernel_size, num_filters, strides, auto_pad, padding_mode, use_bias, dilation,
-                                     groups, False, name, filter_rate, **kwargs)
+                                     groups, False, name, depth_multiplier, **kwargs)
         self.activation = get_activation(activation)
 
         self.padding = kwargs.get('padding', None)
@@ -460,15 +467,16 @@ class Conv2d(_ConvNd):
         elif len(self.padding) == 2:
             pad_h = self.padding[0] * 2
             pad_w = self.padding[1] * 2
-        if self.padding_mode == 'circular':
-            self.padding = ((pad_h + 1) // 2, pad_h // 2, (pad_w + 1) // 2, pad_w // 2)
-        else:
-            self.padding = (pad_h // 2, pad_h - pad_h // 2, pad_w // 2, pad_w - pad_w // 2)
+
+        self.padding= (pad_h // 2,pad_w // 2)
 
     def conv2d_forward(self, x):
-        if len(self.padding)!=4:
-            self.get_padding(x.size()[1:])
-        x = F.pad(x, self.padding, mode='constant' if self.padding_mode=='zero' else self.padding_mode)
+
+        if self.padding_mode == 'circular':
+            expanded_padding = ((self.padding[1] + 1) // 2, self.padding[1] // 2, (self.padding[0] + 1) // 2, self.padding[0] // 2)
+            x=F.pad(input, expanded_padding, mode='circular')
+        else:
+            x = F.pad(x, (self.padding[1] , self.padding[1], self.padding[0] , self.padding[0] ), mode='constant' if self.padding_mode == 'zero' else self.padding_mode)
         return F.conv2d(x, self.weight, self.bias, self.strides, _pair(0), self.dilation, self.groups)
 
     def forward(self, *x):
@@ -481,12 +489,12 @@ class Conv2d(_ConvNd):
 
 class Conv3d(_ConvNd):
     def __init__(self, kernel_size, num_filters=None, strides=1, auto_pad=True, padding_mode='zero', activation=None,
-                 use_bias=False, dilation=1, groups=1, name='', filter_rate=None, **kwargs):
+                 use_bias=False, dilation=1, groups=1, name='', depth_multiplier=None, **kwargs):
         kernel_size = _triple(kernel_size)
         strides = _triple(strides)
         dilation = _triple(dilation)
         super(Conv3d, self).__init__(kernel_size, num_filters, strides, auto_pad, padding_mode, use_bias, dilation,
-                                     groups, False, name, filter_rate, **kwargs)
+                                     groups, False, name, depth_multiplier, **kwargs)
         self.activation = get_activation(activation)
         if self.padding is not None:
             self.padding = _triple(self.padding)
@@ -520,8 +528,7 @@ class Conv3d(_ConvNd):
         if self.padding_mode == 'circular':
             self.padding = ((pad_z + 1) // 2, pad_z // 2, (pad_h + 1) // 2, pad_h // 2, (pad_w + 1) // 2, pad_w // 2)
         else:
-            self.padding = (
-            pad_z // 2, pad_z - pad_z // 2, pad_h // 2, pad_h - pad_h // 2, pad_w // 2, pad_w - pad_w // 2)
+            self.padding = (pad_z // 2, pad_h // 2, pad_w // 2)
 
     def conv3d_forward(self, x):
         x = F.pad(x, self.padding, mode='constant' if self.padding_mode=='zero' else self.padding_mode)
@@ -537,12 +544,12 @@ class Conv3d(_ConvNd):
 
 class TransConv1d(_ConvNd):
     def __init__(self, kernel_size, num_filters=None, strides=1, auto_pad=True, padding_mode='zero', activation=None,
-                 use_bias=False, dilation=1, groups=1, name='', filter_rate=None, **kwargs):
+                 use_bias=False, dilation=1, groups=1, name='', depth_multiplier=None, **kwargs):
         kernel_size = _single(kernel_size)
         strides = _single(strides)
         dilation = _single(dilation)
         super(TransConv1d, self).__init__(kernel_size, num_filters, strides, auto_pad, padding_mode, use_bias, dilation,
-                                          groups, True, name, filter_rate, **kwargs)
+                                          groups, True, name, depth_multiplier, **kwargs)
         self.activation = get_activation(activation)
 
         if 'padding' in kwargs:
@@ -577,13 +584,13 @@ class TransConv1d(_ConvNd):
 
 class TransConv2d(_ConvNd):
     def __init__(self, kernel_size, num_filters=None, strides=1, auto_pad=True, padding_mode='zero', activation=None,
-                 use_bias=False, dilation=1, groups=1, name='', filter_rate=None, **kwargs):
+                 use_bias=False, dilation=1, groups=1, name='', depth_multiplier=None, **kwargs):
         kernel_size = _pair(kernel_size)
         strides = _pair(strides)
         dilation = _pair(dilation)
 
         super(TransConv2d, self).__init__(kernel_size, num_filters, strides, auto_pad, padding_mode, use_bias, dilation,
-                                          groups, True, name, filter_rate, **kwargs)
+                                          groups, True, name, depth_multiplier, **kwargs)
         self.activation = get_activation(activation)
 
         if self.padding is not None:
@@ -623,12 +630,12 @@ class TransConv2d(_ConvNd):
 
 class TransConv3d(_ConvNd):
     def __init__(self, kernel_size, num_filters=None, strides=1, auto_pad=True, padding_mode='zero', activation=None,
-                 use_bias=False, dilation=1, groups=1, name='', filter_rate=None, **kwargs):
+                 use_bias=False, dilation=1, groups=1, name='', depth_multiplier=None, **kwargs):
         kernel_size = _triple(kernel_size)
         strides = _triple(strides)
         dilation = _triple(dilation)
         super(TransConv3d, self).__init__(kernel_size, num_filters, strides, auto_pad, padding_mode, use_bias, dilation,
-                                          groups, True, name, filter_rate, **kwargs)
+                                          groups, True, name, depth_multiplier, **kwargs)
         if 'padding' in kwargs:
             self.padding = _triple(kwargs.get('padding'))
             self.auto_pad = False
@@ -762,7 +769,7 @@ class SeparableConv3d(Layer):
             self.conv1 = DepthwiseConv3d(kernel_size=self.kernel_size, depth_multiplier=self.depth_multiplier,
                                          strides=self.strides, auto_pad=self.auto_pad, padding_mode=self.padding_mode,
                                          dilation=self.dilation, groups=self.input_filters, bias=self.use_bias)
-            self.pointwise = Conv3d(kernel_size=(1, 1, 1), filter_rate=1, strides=1, use_bias=self.use_bias, dilation=1,
+            self.pointwise = Conv3d(kernel_size=(1, 1, 1), depth_multiplier=1, strides=1, use_bias=self.use_bias, dilation=1,
                                     groups=1)
 
             self.to(self.device)
@@ -779,12 +786,11 @@ class SeparableConv3d(Layer):
 
 class DepthwiseConv1d(Layer):
     def __init__(self, kernel_size, depth_multiplier=1, strides=1, auto_pad=True, padding_mode='zero', activation=None,
-                 use_bias=False, dilation=1, groups=1, name='', filter_rate=None, **kwargs):
+                 use_bias=False, dilation=1, groups=1, name='', **kwargs):
         super(DepthwiseConv1d, self).__init__()
         self.kernel_size = _single(kernel_size)
         self.num_filters = kwargs.get('num_filters')
-        if self.num_filters is None and filter_rate is not None:
-            self.filter_rate = filter_rate
+        self.depth_multiplier = depth_multiplier
         self.depth_multiplier = 1
         self.dilation = _single(dilation)
         self.strides = _single(strides)
@@ -826,83 +832,6 @@ class DepthwiseConv1d(Layer):
         return x
 
 
-class DepthwiseConv1d(Layer):
-    def __init__(self, kernel_size, depth_multiplier=1, strides=1, auto_pad=True, padding_mode='zero', activation=None,
-                 use_bias=False, dilation=1, name='', **kwargs):
-        super(DepthwiseConv1d, self).__init__()
-        self.kernel_size = _single(kernel_size)
-        self.num_filters = kwargs.get('num_filters')
-        self.depth_multiplier = depth_multiplier
-
-        self.dilation = _single(dilation)
-        self.strides = _single(strides)
-        self.use_bias = use_bias
-        self.padding = kwargs.get('padding')  # padding if padding is not None else 0in_channel
-        if self.padding is not None:
-            self.auto_pad = None
-            self.padding = _single(self.padding)
-        else:
-            self.padding = _single(0)
-            self.auto_pad = auto_pad
-        self.padding_mode = padding_mode
-
-        self.activation = get_activation(activation)
-        self.conv1 = None
-        self._built = False
-
-    def get_padding(self, input_shape):
-        pad_h = 0
-        pad_w = 0
-        if self.auto_pad == True:
-            iw = to_list(input_shape)[-1]
-            kw = self.kernel_size[-1]
-            sw = self.strides[-1]
-            dw = self.dilation[-1]
-            ow = math.ceil(iw / sw)
-            pad_w = max(round((ow - 1) * sw + (kw - 1) * dw + 1 - iw), 0)
-            if pad_w % 2 == 1 and sw > 1:
-                pad_w += 1
-        elif len(self.padding) == 1:
-            pad_w = self.padding[1] * 2
-        if self.padding_mode == 'circular':
-            self.padding = ( (pad_w + 1) // 2, pad_w // 2)
-        else:
-            self.padding = ( pad_w // 2, pad_w - pad_w // 2)
-
-    def build(self, input_shape):
-        if self._built == False:
-            self.num_filters = self.input_filters * self.depth_multiplier if self.depth_multiplier is not None else self.num_filters
-            self.groups = self.input_filters
-            if self.input_filters % self.groups != 0:
-                raise ValueError('in_channels must be divisible by groups')
-
-            self.get_padding(input_shape)
-
-            self.weight = Parameter(
-                torch.Tensor(int(self.num_filters), int(self.input_filters) // self.groups, *self.kernel_size))  #
-
-            init.kaiming_uniform_(self.weight, a=math.sqrt(5))
-
-            if self.use_bias:
-                self.bias = Parameter(torch.Tensor(int(self.num_filters)))
-                init.zeros_(self.bias)
-            else:
-                self.register_parameter('bias', None)
-            self.to(self.device)
-            self._built = True
-
-    def conv1d_forward(self, x):
-        if len(self.padding)!=2:
-            self.get_padding(x.size()[1:])
-        x = F.pad(x, self.padding, mode='constant' if self.padding_mode=='zero' else self.padding_mode)
-        return F.conv1d(x, self.weight, self.bias, self.strides, _single(0), self.dilation, self.groups)
-
-    def forward(self, *x):
-        x = enforce_singleton(x)
-        x = self.conv1d_forward(x)
-        if self.activation is not None:
-            x = self.activation(x)
-        return x
 
 
 
@@ -947,15 +876,14 @@ class DepthwiseConv2d(Layer):
         elif len(self.padding) == 2:
             pad_h = self.padding[0] * 2
             pad_w = self.padding[1] * 2
-        if self.padding_mode == 'circular':
-            self.padding = ((pad_h + 1) // 2, pad_h // 2, (pad_w + 1) // 2, pad_w // 2)
-        else:
-            self.padding = (pad_h // 2, pad_h - pad_h // 2, pad_w // 2, pad_w - pad_w // 2)
+        self.padding=(pad_h//2,pad_w//2)
+
 
     def build(self, input_shape):
         if self._built == False:
-            self.num_filters = self.input_filters * self.depth_multiplier if self.depth_multiplier is not None else self.num_filters
-            self.groups = self.input_filters
+            self.num_filters = self.input_filters * self.depth_multiplier if self.depth_multiplier is not None  and self.num_filters is None else self.num_filters
+            self.groups = self.input_filters if self.depth_multiplier>=1 else int(self.input_filters *self.depth_multiplier)
+
             if self.input_filters % self.groups != 0:
                 raise ValueError('in_channels must be divisible by groups')
 
@@ -988,9 +916,12 @@ class DepthwiseConv2d(Layer):
             self._built = True
 
     def conv2d_forward(self, x):
-        if len(self.padding)!=4:
-            self.get_padding(x.size()[1:])
-        x = F.pad(x, self.padding, mode='constant' if self.padding_mode=='zero' else self.padding_mode)
+        if self.padding_mode == 'circular':
+            expanded_padding = ((self.padding[1] + 1) // 2, self.padding[1] // 2, (self.padding[0] + 1) // 2, self.padding[0] // 2)
+            x = F.pad(input, expanded_padding, mode='circular')
+        else:
+            x = F.pad(x, (self.padding[1], self.padding[1], self.padding[0], self.padding[0]),mode='constant' if self.padding_mode == 'zero' else self.padding_mode)
+
         return F.conv2d(x, self.weight, self.bias, self.strides, _pair(0), self.dilation, self.groups)
 
     def forward(self, *x):
@@ -1051,12 +982,12 @@ class DepthwiseConv3d(Layer):
 
 class DeformConv2d(Layer):
     def __init__(self, kernel_size, num_filters=None, strides=1, offset_group=2, auto_pad=True, padding_mode='zero',
-                 activation=None, use_bias=False, dilation=1, groups=1, name='', filter_rate=None, **kwargs):
+                 activation=None, use_bias=False, dilation=1, groups=1, name='', depth_multiplier=None, **kwargs):
         super(DeformConv2d, self).__init__()
         self.kernel_size = _pair(kernel_size)
         self.num_filters = kwargs.get('num_filters')
-        if self.num_filters is None and filter_rate is not None:
-            self.filter_rate = filter_rate
+        if self.num_filters is None and depth_multiplier is not None:
+            self.depth_multiplier = depth_multiplier
 
         self.dilation = _pair(dilation)
         self.strides = _pair(strides)
@@ -1152,12 +1083,12 @@ class DeformConv2d(Layer):
 class GcdConv1d(Layer):
     def __init__(self, kernel_size, num_filters=None, strides=1, auto_pad=True, padding_mode='zero', activation=None,
                  use_bias=False, dilation=1, divisor_rank=0, self_norm=True, is_shuffle=False, name='',
-                 filter_rate=None, **kwargs):
+                 depth_multiplier=None, **kwargs):
         super(GcdConv1d, self).__init__()
         self.kernel_size = _single(kernel_size)
         self.num_filters = num_filters
-        if self.num_filters is None and filter_rate is not None:
-            self.filter_rate = filter_rate
+        if self.num_filters is None and depth_multiplier is not None:
+            self.depth_multiplier = depth_multiplier
         self.strides = _single(strides)
         self.auto_pad = auto_pad
         self.padding = 0
@@ -1278,12 +1209,12 @@ class GcdConv1d(Layer):
 class GcdConv2d(Layer):
     def __init__(self, kernel_size, num_filters=None, strides=1, auto_pad=True, padding_mode='zero', activation=None,
                  use_bias=False, dilation=1, divisor_rank=0, self_norm=True, is_shuffle=False, crossgroup_fusion=True,
-                 name='', filter_rate=None, **kwargs):
+                 name='', depth_multiplier=None, **kwargs):
         super(GcdConv2d, self).__init__()
         self.kernel_size = _pair(kernel_size)
         self.num_filters = num_filters
-        if self.num_filters is None and filter_rate is not None:
-            self.filter_rate = filter_rate
+        if self.num_filters is None and depth_multiplier is not None:
+            self.depth_multiplier = depth_multiplier
         self.input_filters = None
         self.strides = _pair(strides)
         self.auto_pad = auto_pad
@@ -1430,12 +1361,12 @@ class GcdConv2d(Layer):
 
 class GcdConv2d_1(Layer):
     def __init__(self, kernel_size, num_filters=None, strides=1, auto_pad=True, padding_mode='zero', activation=None,
-                 use_bias=False, divisor_rank=0, dilation=1, self_norm=True, name='', filter_rate=None, **kwargs):
+                 use_bias=False, divisor_rank=0, dilation=1, self_norm=True, name='', depth_multiplier=None, **kwargs):
         super(GcdConv2d_1, self).__init__(name=name)
         self.kernel_size = kernel_size
         self.num_filters = num_filters
-        if self.num_filters is None and filter_rate is not None:
-            self.filter_rate = filter_rate
+        if self.num_filters is None and depth_multiplier is not None:
+            self.depth_multiplier = depth_multiplier
         self.input_filters = 3
         self.strides = _pair(strides)
         self.auto_pad = auto_pad
@@ -1877,3 +1808,23 @@ class AlphaDropout(Layer):
 
     def extra_repr(self):
         return 'p={}, inplace={}'.format(self.dropout_rate, self.inplace)
+
+
+
+class SingleImageLayer(Layer):
+    def __init__(self, image,is_recursive=False,name=''):
+        super(SingleImageLayer, self).__init__(name=name)
+        if isinstance(image,(np.ndarray,torch.Tensor)):
+            self.origin_image = to_tensor(image).squeeze()
+            self.input_shape = image.shape[1:]
+
+    def build(self, input_shape):
+        if self._built == False:
+            self.weight = Parameter(self.origin_image.clone(), requires_grad=True)
+            self.input_filters = input_shape[0]
+            self._built = True
+    def forward(self,x):
+        return self.weight.data.unsqueeze(0)
+
+    def extra_repr(self):
+        return 'is_recursive={0}'.format(self.is_recursive)
