@@ -1,13 +1,12 @@
 import platform
 import tkinter
-import matplotlib
-# if platform.system() not in ['Linux', 'Darwin'] and not platform.system().startswith('CYGWIN'):
-#     matplotlib.use('TKAgg')
-import matplotlib.pyplot as plt
-from matplotlib.collections import PolyCollection
 from .ipython_utils import is_in_ipython
 if is_in_ipython():
     from IPython import display
+import matplotlib
+matplotlib.use('Qt5Agg' if not is_in_ipython() else 'TkAgg')
+import matplotlib.pyplot as plt
+from matplotlib.collections import PolyCollection
 
 import itertools
 import numpy as np
@@ -20,28 +19,45 @@ __all__ = ['tile_rgb_images', 'loss_metric_curve', 'steps_histogram']
 
 def tile_rgb_images(*imgs, row=3, save_path=None, imshow=False):
     make_dir_if_need(save_path)
-    fig = plt.gcf()
-    fig.set_size_inches(len(imgs) * 2, row * 2)
-    plt.clf()
-    plt.ion()  # is not None:
     suffix = get_time_suffix()
+    if len(imgs)==1 and row==1:
+        img = array2image(imgs[0][0])
+        filename = save_path.format(suffix)
+        img.save(filename)
+        plt.imshow(img)
+        if imshow:
+            if is_in_ipython():
+                plt.axis("off")
+                display.display(plt.gcf())
+                plt.close()
+            else:
+                plt.axis("off")
+                plt.imshow(img, interpolation="nearest", animated=True)
+                plt.ioff()
+                plt.gcf().show()
+    else:
+        fig = plt.gcf()
+        fig.set_size_inches(len(imgs) * 2, row * 2)
+        plt.clf()
+        plt.ion()  # is not None:
 
-    for m in range(row * len(imgs)):
-        plt.subplot(row, len(imgs), m + 1)
-        img = array2image((imgs[int(m % len(imgs))][int(m // len(imgs))]))
-        plt.imshow(img, interpolation="nearest", animated=True)
-        plt.axis("off")
-    filename = save_path.format(suffix)
-    plt.savefig(filename, bbox_inches='tight')
-    if imshow == True:
-        plSize = fig.get_size_inches()
-        fig.set_size_inches((int(round(plSize[0] * 0.75, 0)), int(round(plSize[1] * 0.75, 0))))
-        if is_in_ipython():
-            display.display(plt.gcf())
-            plt.close(fig)
-        else:
-            plt.ioff()
-            plt.gcf().show()
+
+        for m in range(row * len(imgs)):
+            plt.subplot(row, len(imgs), m + 1)
+            img = array2image((imgs[int(m % len(imgs))][int(m // len(imgs))]))
+            plt.imshow(img, interpolation="nearest", animated=True)
+            plt.axis("off")
+        filename = save_path.format(suffix)
+        plt.savefig(filename, bbox_inches='tight')
+        if imshow == True:
+            plSize = fig.get_size_inches()
+            fig.set_size_inches((int(round(plSize[0] * 0.75, 0)), int(round(plSize[1] * 0.75, 0))))
+            if is_in_ipython():
+                display.display(plt.gcf())
+                plt.close(fig)
+            else:
+                plt.ioff()
+                plt.gcf().show()
 
 
 def loss_metric_curve(losses, metrics, sample_collected=None, legend=None, calculate_base='epoch', max_iteration=None,
@@ -62,7 +78,7 @@ def loss_metric_curve(losses, metrics, sample_collected=None, legend=None, calcu
             plt.plot(collected_samples, losses['total_losses'])
         else:
             plt.plot(losses['total_losses'])
-        plt.legend(['loss'], loc='upper right')
+        plt.legend(['loss'], loc='upper left')
     elif isinstance(losses, list):
         for item in losses:
             if len(collected_samples) > 0:
@@ -108,6 +124,7 @@ def loss_metric_curve(losses, metrics, sample_collected=None, legend=None, calcu
     plt.ylabel('metrics')
     plt.xlabel(calculate_base)
 
+
     if max_iteration is not None:
         plt.xlim(0, max_iteration)
 
