@@ -32,7 +32,7 @@ from ..data.image_common import *
 from ..data.utils import download_model_from_google_drive
 
 
-__all__ = []
+__all__ = ['DeeplabV3_plus','DeeplabV3']
 
 _session = get_session()
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
@@ -52,7 +52,7 @@ if not os.path.exists(dirname):
 
 
 
-def DeepLabHead(classes=20, atrous_rates=(6, 12, 18),num_filters=256):
+def DeepLabHead(classes=20, atrous_rates=(6, 12, 18,24),num_filters=256):
     return Sequential(
         ASPP(atrous_rates,num_filters=num_filters),
         Conv2d_Block((3,3),num_filters,auto_pad=True,use_bias=False,activation='relu',normalization='batch'),
@@ -63,7 +63,7 @@ def DeepLabHead(classes=20, atrous_rates=(6, 12, 18),num_filters=256):
 
 def ASPPPooling(num_filters,size):
     return Sequential(AdaptiveAvgPool2d((1,1)),
-                      Conv2d((1,1),num_filters,strides=1,use_bias=False,activation='relu'),
+                      Conv2d((1,1),num_filters,strides=1,use_bias=False,activation=None),
                       Upsampling2d(size=(size[-2], size[-1]),mode='bilinear', align_corners=False))
 
 
@@ -73,14 +73,10 @@ class ASPP(Layer):
         super(ASPP, self).__init__()
         self.num_filters=num_filters
         self.convs = ShortCut2d( mode=ShortcutMode.concate)
-        self.convs.add_module('conv1',Conv2d_Block((1,1),num_filters=num_filters,strides=1,use_bias=False,activation='relu',normalization='batch'))
+        self.convs.add_module('conv1',Conv2d_Block((1,1),num_filters=num_filters,strides=1,use_bias=False,activation=None,normalization='batch'))
 
-
-        rate1, rate2, rate3 = tuple(atrous_rates)
-
-        self.convs.add_module('aspp_dilation1',Conv2d_Block((1,1),num_filters=num_filters,strides=1,use_bias=False,activation='relu',normalization='batch',dilation=rate1))
-        self.convs.add_module('aspp_dilation2',Conv2d_Block((1,1),num_filters=num_filters,strides=1,use_bias=False,activation='relu',normalization='batch',dilation=rate2))
-        self.convs.add_module('aspp_dilation3',Conv2d_Block((1,1),num_filters=num_filters,strides=1,use_bias=False,activation='relu',normalization='batch',dilation=rate3))
+        for i in range(len(atrous_rates)):
+            self.convs.add_module('aspp_dilation{0}'.format(i),Conv2d_Block((3,3),num_filters=num_filters,strides=1,use_bias=False,activation=None,normalization='batch',dilation=atrous_rates[i]))
 
         self.project =Conv2d_Block( (1,1),num_filters,strides=1,use_bias=False, bias=False,activation='relu',normalization='batch',dilation=1,dropout_rate=0.5)
 
@@ -100,7 +96,7 @@ class ASPP(Layer):
 
 def DeeplabV3(backbond,
              input_shape=(3,224,224),
-             classes=10,
+             classes=20,
              **kwargs):
     input_shape=tuple(input_shape)
     deeplab=Sequential(name='deeplabv3')
@@ -111,3 +107,56 @@ def DeeplabV3(backbond,
     model = ImageSegmentationModel(input_shape=input_shape, output=deeplab)
     return model
 
+
+
+class _DeeplabV3_plus(Layer):
+    def __init__(self, backbond, input_shape=(3,224,224), atrous_rates=(6, 12, 18, 24), num_filters=256, classes=20):
+        super(_DeeplabV3_plus, self).__init__()
+        moduals=list(backbond.children())
+        low_level_idx=-1
+        high_level_idx=-1
+        for i in range(len(moduals)):
+            if low_level_idx<0 and moduals[i].output_shape[-1]==backbond.input_shape[-1]//8:
+                low_level_idx=i
+
+            if high_level_idx<0 and moduals[i].output_shape[-1]==backbond.input_shape[-1]//32:
+                high_level_idx=i
+                break
+        self.num_filters=num_filters
+        self.classes=classes
+        self.atrous_rates=atrous_rates
+        self.backbond1=Sequential(backbond[:low_level_idx])
+        self.backbond2 = Sequential(backbond[low_level_idx:high_level_idx])
+        self.aspp=ASPP(atrous_rates=self.atrous_rates,num_filters=self.num_filters)
+        self.low_level_conv=Conv2d_Block((1,1),num_filters=int(48*self.num_filters/256),strides=1,use_bias=False,activation='leaky_relu',normalization='batch')
+        self.decoder=Sequential(
+            DepthwiseConv2d_Block((3,3),depth_multiplier=0.5,strides=1,use_bias=False,activation='leaky_relu',normalization='batch',dropout_rate=0.5),
+            DepthwiseConv2d_Block((3,3),depth_multiplier=1,strides=1,use_bias=False,activation='leaky_relu',normalization='batch',dropout_rate=0.1),
+            Conv2d((1, 1), num_filters=self.classes, strides=1, use_bias=False, activation='sigmoid'),
+
+        )
+
+    def forward(self, *x):
+        x = enforce_singleton(x)
+        low_level_feature=self.backbond1(x)
+        high_level_feature = self.backbond2(low_level_feature)
+        x=self.aspp(high_level_feature)
+        x=F.interpolate(x, None, (4.0,4.0),mode='bilinear', align_corners=True)
+        low_level_feature=self.low_level_conv(low_level_feature)
+        x=torch.cat([x,low_level_feature],dim=1)
+        x=self.decoder(x)
+        x = F.interpolate(x, None, (4.0, 4.0), mode='bilinear', align_corners=True)
+        return x
+
+
+
+def DeeplabV3_plus(backbond,
+             input_shape=(3,224,224),
+             atrous_rates = (6, 12, 18, 24),
+             num_filters = 256,
+             classes=20,
+             **kwargs):
+    deeplab=_DeeplabV3_plus(backbond=backbond,input_shape=input_shape,atrous_rates=atrous_rates,num_filters=num_filters,classes=classes)
+    deeplab.name='DeeplabV3_plus'
+    model = ImageSegmentationModel(input_shape=input_shape, output=deeplab)
+    return model
