@@ -209,8 +209,8 @@ def EfficientNet(width_coefficient,
     if include_top:
         if dropout_rate > 0:
             efficientnet.add_module('top_dropout',Dropout(dropout_rate,name='top_dropout'))
-        efficientnet.add_module('fc',Dense(num_classes,activation='softmax',name='fc'))
-
+        efficientnet.add_module('fc',Dense(num_classes,activation=None,name='fc'))
+        efficientnet.add_module('softmax', SoftMax(name='softmax'))
     if isinstance(default_size,int):
         default_size=default_size,
     if len(default_size)==1:
@@ -246,7 +246,7 @@ def EfficientNetB0(include_top=True,
         else:
             if classes!=1000:
 
-                new_fc = Dense(classes, activation='softmax', name='fc')
+                new_fc = Dense(classes, activation=None, name='fc')
                 new_fc.input_shape=recovery_model.fc.input_shape
                 recovery_model.fc=new_fc
         recovery_model.to(_device)
@@ -276,7 +276,7 @@ def EfficientNetB1(include_top=True,
         else:
             if classes!=1000:
 
-                new_fc = Dense(classes, activation='softmax', name='fc')
+                new_fc = Dense(classes, activation=None, name='fc')
                 new_fc.input_shape=recovery_model.fc.input_shape
                 recovery_model.fc=new_fc
         effb1.model=recovery_model
@@ -303,7 +303,7 @@ def EfficientNetB2(include_top=True,
         else:
             if classes!=1000:
 
-                new_fc = Dense(classes, activation='softmax', name='fc')
+                new_fc = Dense(classes, activation=None, name='fc')
                 new_fc.input_shape=recovery_model.fc.input_shape
                 recovery_model.fc=new_fc
         effb2.model=recovery_model
@@ -330,7 +330,7 @@ def EfficientNetB3(include_top=True,
         else:
             if classes!=1000:
 
-                new_fc = Dense(classes, activation='softmax', name='fc')
+                new_fc = Dense(classes, activation=None, name='fc')
                 new_fc.input_shape=recovery_model.fc.input_shape
                 recovery_model.fc=new_fc
         effb3.model=recovery_model
@@ -357,7 +357,7 @@ def EfficientNetB4(include_top=True,
         else:
             if classes!=1000:
 
-                new_fc = Dense(classes, activation='softmax', name='fc')
+                new_fc = Dense(classes, activation=None, name='fc')
                 new_fc.input_shape=recovery_model.fc.input_shape
                 recovery_model.fc=new_fc
         effb4.model=recovery_model
@@ -384,7 +384,7 @@ def EfficientNetB5(include_top=True,
         else:
             if classes!=1000:
 
-                new_fc = Dense(classes, activation='softmax', name='fc')
+                new_fc = Dense(classes, activation=None, name='fc')
                 new_fc.input_shape=recovery_model.fc.input_shape
                 recovery_model.fc=new_fc
         effb5.model=recovery_model
@@ -411,7 +411,7 @@ def EfficientNetB6(include_top=True,
         else:
             if classes!=1000:
 
-                new_fc = Dense(classes, activation='softmax', name='fc')
+                new_fc = Dense(classes, activation=None, name='fc')
                 new_fc.input_shape=recovery_model.fc.input_shape
                 recovery_model.fc=new_fc
         effb6.model=recovery_model
@@ -438,7 +438,7 @@ def EfficientNetB7(include_top=True,
         else:
             if classes!=1000:
 
-                new_fc = Dense(classes, activation='softmax', name='fc')
+                new_fc = Dense(classes, activation=None, name='fc')
                 new_fc.input_shape=recovery_model.fc.input_shape
                 recovery_model.fc=new_fc
         effb7.model=recovery_model
