@@ -42,10 +42,10 @@ def resnet_block(num_filters=64, strides=1, activation='leaky_relu', normalizati
                  dilation=1, name=''):
     kernal = 1 if strides == 1 else 3
     return [ShortCut2d(Sequential(
-        Conv2d_Block((3, 3), filter_rate=0.5, strides=1, auto_pad=True, padding_mode='replicate',
+        Conv2d_Block((3, 3), depth_multiplier=0.5, strides=1, auto_pad=True, padding_mode='replicate',
                      use_spectral=use_spectral, normalization=normalization, activation=activation, use_bias=False,
                      dilation=dilation, name=name + '_0_conv'),
-        Conv2d_Block((1, 1), filter_rate=2, strides=1, auto_pad=True, padding_mode='replicate',
+        Conv2d_Block((1, 1), depth_multiplier=2, strides=1, auto_pad=True, padding_mode='replicate',
                      use_spectral=use_spectral, normalization=normalization, activation=activation, use_bias=False,
                      name=name + '_1_conv')), Identity(), activation=activation, name=name),
             Conv2d_Block((kernal, kernal), num_filters=num_filters, strides=strides, auto_pad=True,
@@ -60,7 +60,7 @@ def separable_resnet_block(num_filters=64, strides=1, activation='leaky_relu', n
         SeparableConv2d_Block((3, 3), depth_multiplier=1, strides=1, auto_pad=True, padding_mode='replicate',
                               use_spectral=use_spectral, normalization=normalization, activation=activation,
                               use_bias=False, dilation=dilation, name=name + '_0_conv'),
-        Conv2d_Block((1, 1), filter_rate=2, strides=1, auto_pad=True, padding_mode='replicate',
+        Conv2d_Block((1, 1), depth_multiplier=2, strides=1, auto_pad=True, padding_mode='replicate',
                      use_spectral=use_spectral, normalization=normalization, activation=activation, use_bias=False,
                      name=name + '_1_conv')), Identity(), activation=activation, name=name),
             Conv2d_Block((kernal, kernal), num_filters=num_filters, strides=strides, auto_pad=True,
@@ -72,10 +72,10 @@ def bottleneck_block(num_filters=64, strides=1, reduce=4, activation='leaky_relu
                      use_spectral=False, dilation=1, name=''):
     shortcut = Conv2d_Block((3, 3), num_filters=num_filters, strides=strides, auto_pad=True, padding_mode='zero',
                             normalization=normalization, activation=None, name=name + '_downsample')
-    return ShortCut2d(Sequential(Conv2d_Block((1, 1), filter_rate=1, strides=1, auto_pad=True, padding_mode='replicate',
+    return ShortCut2d(Sequential(Conv2d_Block((1, 1), depth_multiplier=1, strides=1, auto_pad=True, padding_mode='replicate',
                                               use_spectral=use_spectral, normalization=normalization,
                                               activation=activation, use_bias=False, name=name + '_0_conv'),
-                                 Conv2d_Block((3, 3), filter_rate=1 / reduce, strides=strides, auto_pad=True,
+                                 Conv2d_Block((3, 3), depth_multiplier=1 / reduce, strides=strides, auto_pad=True,
                                               padding_mode='replicate', use_spectral=use_spectral,
                                               normalization=normalization, activation=activation, use_bias=False,
                                               dilation=dilation, name=name + '_1_conv'),
@@ -148,7 +148,7 @@ def gan_builder(noise_shape=100, image_width=256, upsample_mode='nearest', gener
 
             if upsample_mode == UpsampleMode.transpose.value:
                 layers.append(
-                    TransConv2d_Block((3, 3), filter_rate=1, strides=scale, auto_pad=True, padding_mode='replicate',
+                    TransConv2d_Block((3, 3), depth_multiplier=1, strides=scale, auto_pad=True, padding_mode='replicate',
                                       use_spectral=use_spectral, use_bias=False, activation=activation,
                                       normalization=generator_norm, dilation=dilation,
                                       name='transconv_block{0}'.format(i)))
