@@ -2,10 +2,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 import matplotlib
-import platform
-if platform.system() not in ['Linux', 'Darwin'] and not platform.system().startswith('CYGWIN'):
-    matplotlib.use('TKAgg')
-import matplotlib.pyplot as plt
+
 from ..misc.ipython_utils import is_in_ipython
 if is_in_ipython():
     from IPython import display
@@ -40,6 +37,7 @@ from ..layers.pytorch_normalizations import get_normalization
 from ..layers.pytorch_blocks import *
 from ..layers.pytorch_pooling import *
 from ..optims.pytorch_trainer import *
+from ..optims.pytorch_trainer import ImageDetectionModel
 from ..data.image_common import *
 from ..data.utils import download_model_from_google_drive
 from ..data.bbox_common import clip_boxes_to_image,nms
