@@ -21,6 +21,7 @@ from itertools import repeat
 from ..backend.pytorch_ops import *
 from ..backend.common import *
 from ..backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential,get_signature
+from ..data.bbox_common import xywh_to_xyxy,xyxy_to_xywh
 from ..layers.pytorch_layers import *
 from ..layers.pytorch_activations import get_activation, Identity, Relu
 from ..layers.pytorch_normalizations import get_normalization
@@ -28,6 +29,7 @@ from ..layers.pytorch_blocks import *
 from ..layers.pytorch_pooling import *
 from ..optims.pytorch_trainer import *
 from ..data.image_common import *
+
 from ..data.utils import download_model_from_google_drive
 
 __all__ = ['Mobile_RFBnet',  'generate_priors']
@@ -114,27 +116,27 @@ def center_form_to_corner_form(locations):
 
 def basic_rfb(num_filters, scale=0.1):
     return ShortCut2d(Sequential(ShortCut2d(Sequential(
-        Conv2d_Block((1, 1), filter_rate=0.125, strides=1, groups=1, auto_pad=True, use_bias=False, activation=None,
+        Conv2d_Block((1, 1), depth_multiplier=0.125, strides=1, groups=1, auto_pad=True, use_bias=False, activation=None,
                      normalization='batch'),
-        Conv2d_Block((3, 3), filter_rate=2, strides=1, groups=1, auto_pad=True, use_bias=False, activation='relu',
+        Conv2d_Block((3, 3), depth_multiplier=2, strides=1, groups=1, auto_pad=True, use_bias=False, activation='relu',
                      normalization='batch'),
-        Conv2d_Block((3, 3), filter_rate=1, strides=1, groups=1, auto_pad=True, use_bias=False, dilation=2,
+        Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, auto_pad=True, use_bias=False, dilation=2,
                      activation=None, normalization='batch'), name='branch1'),
 
         Sequential(
-            Conv2d_Block((1, 1), filter_rate=0.125, strides=1, groups=1, auto_pad=True, use_bias=False, activation=None,
+            Conv2d_Block((1, 1), depth_multiplier=0.125, strides=1, groups=1, auto_pad=True, use_bias=False, activation=None,
                          normalization='batch'),
-            Conv2d_Block((3, 3), filter_rate=2, strides=1, groups=1, auto_pad=True, use_bias=False, activation='relu',
+            Conv2d_Block((3, 3), depth_multiplier=2, strides=1, groups=1, auto_pad=True, use_bias=False, activation='relu',
                          normalization='batch'),
-            Conv2d_Block((3, 3), filter_rate=1, strides=1, groups=1, auto_pad=True, use_bias=False, dilation=3,
+            Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, auto_pad=True, use_bias=False, dilation=3,
                          activation=None, normalization='batch'), name='branch1'), Sequential(
-            Conv2d_Block((1, 1), filter_rate=0.125, strides=1, groups=1, auto_pad=True, use_bias=False, activation=None,
+            Conv2d_Block((1, 1), depth_multiplier=0.125, strides=1, groups=1, auto_pad=True, use_bias=False, activation=None,
                          normalization='batch'),
-            Conv2d_Block((3, 3), filter_rate=1.5, strides=1, groups=1, auto_pad=True, use_bias=False, activation='relu',
+            Conv2d_Block((3, 3), depth_multiplier=1.5, strides=1, groups=1, auto_pad=True, use_bias=False, activation='relu',
                          normalization='batch'),
-            Conv2d_Block((3, 3), filter_rate=1.34, strides=1, groups=1, auto_pad=True, use_bias=False,
+            Conv2d_Block((3, 3), depth_multiplier=1.34, strides=1, groups=1, auto_pad=True, use_bias=False,
                          activation='relu', normalization='batch'),
-            Conv2d_Block((3, 3), filter_rate=1, strides=1, groups=1, auto_pad=True, use_bias=False, dilation=5,
+            Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, auto_pad=True, use_bias=False, dilation=5,
                          activation=None, normalization='batch'), name='branch2'), mode=ShortcutMode.concate),
         Conv2d_Block((1, 1), num_filters=num_filters, strides=1, groups=1, auto_pad=True, use_bias=False,
                      activation=None, normalization='batch')),
@@ -167,7 +169,7 @@ def tiny_mobile_rfbnet(filter_base=16, num_classes=4):
 
 
 class RFBnet(Layer):
-    def __init__(self, *args, base_filters=16, num_classes=4, iou_threshold=0.3, center_variance=0.1, size_variance=0.2,
+    def __init__(self, *args, base_filters=16, num_classes=4, num_regressors=4,iou_threshold=0.3, center_variance=0.1, size_variance=0.2,
                  name='tiny_mobile_rfbnet', **kwargs):
         """
 
@@ -186,6 +188,7 @@ class RFBnet(Layer):
         self.size_variance = size_variance
 
         self.num_classes = num_classes
+        self.num_regressors=num_regressors
         self.priors = []
         self.define_img_size(640)
 
@@ -195,17 +198,17 @@ class RFBnet(Layer):
                                 Conv2d((1, 1), num_filters=256, strides=1, activation=None, use_bias=True), Relu())
         self.regression_headers = nn.ModuleList([Sequential(
             DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
-            Conv2d((1, 1), num_filters=3 * 4, strides=1, activation=None, use_bias=True)),
+            Conv2d((1, 1), num_filters=3 * self.num_regressors, strides=1, activation=None, use_bias=True)),
 
             Sequential(
                 DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
-                Conv2d((1, 1), num_filters=2 * 4, strides=1, activation=None, use_bias=True)),
+                Conv2d((1, 1), num_filters=2 *self.num_regressors, strides=1, activation=None, use_bias=True)),
 
             Sequential(
                 DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
-                Conv2d((1, 1), num_filters=2 * 4, strides=1, activation=None, use_bias=True)),
+                Conv2d((1, 1), num_filters=2 * self.num_regressors, strides=1, activation=None, use_bias=True)),
 
-            Conv2d((3, 3), num_filters=12, strides=1, auto_pad=True, activation=None), ])
+            Conv2d((3, 3), num_filters=3*self.num_regressors, strides=1, auto_pad=True, activation=None), ])
         self.classification_headers = nn.ModuleList([Sequential(
             DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
             Conv2d((1, 1), num_filters=3 * self.num_classes, strides=1, activation=None, use_bias=True)),
@@ -282,10 +285,13 @@ class RFBnet(Layer):
         if self.training:
             return confidences, locations
         else:
-            confidences = F.softmax(confidences, dim=2)
+            confidences_class = F.softmax(confidences[:,:,2],dim=2)
+            confidences_attr=confidences[:,:,2].sigmoid()
             boxes = convert_locations_to_boxes(locations, self.priors, self.center_variance, self.size_variance)
-            boxes = center_form_to_corner_form(boxes)
-            return confidences, boxes
+            boxes = xywh_to_xyxy(boxes)
+            return confidences_class, boxes,confidences_attr
+
+
 
 
 class SsdDetectionModel(ImageDetectionModel):
@@ -337,8 +343,10 @@ class SsdDetectionModel(ImageDetectionModel):
         Returns:
              picked: a list of indexes of the kept boxes
         """
+        if box_scores is None or len(box_scores)==0:
+            return None, None
         scores = box_scores[:, -1]
-        boxes = box_scores[:, :-1]
+        boxes = box_scores[:, :4]
         picked = []
         # _, indexes = scores.sort(descending=True)
         indexes = np.argsort(scores)
@@ -357,76 +365,168 @@ class SsdDetectionModel(ImageDetectionModel):
             iou = self.iou_of(rest_boxes, np.expand_dims(current_box, axis=0), )
             indexes = indexes[iou <= iou_threshold]
 
-        return box_scores[picked, :]
+        return box_scores[picked, :],picked
 
-    def predict(self, width, height, confidences, boxes, prob_threshold, iou_threshold=0.3, top_k=-1):
+
+    def predict(self, width, height, confidences_class, boxes, confidences_attr, prob_threshold, iou_threshold=0.3, top_k=-1):
         boxes = boxes[0]
-        confidences = confidences[0]
-        picked_box_probs = []
-        picked_labels = []
-        for class_index in range(1, confidences.shape[1]):
-            probs = confidences[:, class_index]
-            mask = probs > prob_threshold
-            probs = probs[mask]
-            if probs.shape[0] == 0:
-                continue
+        confidences_class = confidences_class[0]
+        confidences_attr=confidences_attr[0]
+        picked_box_probs = None
+        picked_labels =None
+
+        probs = confidences_class[:, 1]
+        labels=np.argmax(confidences_class,-1)
+        mask = probs > prob_threshold
+
+        probs = probs[mask]
+        labels = labels[mask]
+        confidences_attr= confidences_attr[mask]
+        if probs.shape[0] > 0:
             subset_boxes = boxes[mask, :]
             box_probs = np.concatenate([subset_boxes, probs.reshape(-1, 1)], axis=1)
-            box_probs = self.hard_nms(box_probs, iou_threshold=iou_threshold, top_k=top_k, )
-            picked_box_probs.append(box_probs)
-            picked_labels.extend([class_index] * box_probs.shape[0])
-        if not picked_box_probs:
-            return np.array([]), np.array([]), np.array([])
+            box_probs, keep = self.hard_nms(box_probs, iou_threshold=iou_threshold, top_k=top_k, )
+            if box_probs is not None:
+                picked_box_probs = box_probs
+                picked_labels = labels[keep]
+                confidences_attr = confidences_attr[keep]
+            else:
+                return None, None, None
+
         picked_box_probs = np.concatenate(picked_box_probs)
         picked_box_probs[:, 0] *= width
         picked_box_probs[:, 1] *= height
         picked_box_probs[:, 2] *= width
         picked_box_probs[:, 3] *= height
-        return picked_box_probs[:, :4].astype(np.int32), np.array(picked_labels), picked_box_probs[:, 4]
+        if picked_box_probs.shape[-1]>=14:
+            picked_box_probs[:, 4] *= width
+            picked_box_probs[:, 5] *= height
+            picked_box_probs[:, 6] *= width
+            picked_box_probs[:, 7] *= height
+            picked_box_probs[:, 8] *= width
+            picked_box_probs[:, 9] *= height
+            picked_box_probs[:, 10] *= width
+            picked_box_probs[:, 11] *= height
+            picked_box_probs[:, 12] *= width
+            picked_box_probs[:, 13] *= height
+
+        return picked_box_probs, picked_labels,confidences_attr
+
+
+    def nms(self,boxes, threshold=0.3):
+        # if there are no boxes, return an empty list
+        if len(boxes) == 0:
+            return []
+
+        # initialize the list of picked indexes
+        pick = []
+
+        # grab the coordinates of the bounding boxes
+        x1 = boxes[:, 0]
+        y1 = boxes[:, 1]
+        x2 = boxes[:, 2]
+        y2 = boxes[:, 3]
+
+        # compute the area of the bounding boxes and sort the bounding
+        # boxes by the bottom-right y-coordinate of the bounding box
+        area = (x2 - x1 + 1) * (y2 - y1 + 1)
+        idxs = np.argsort(y2)
+
+        # keep looping while some indexes still remain in the indexes
+        # list
+        while len(idxs) > 0:
+            # grab the last index in the indexes list, add the index
+            # value to the list of picked indexes, then initialize
+            # the suppression list (i.e. indexes that will be deleted)
+            # using the last index
+            last = len(idxs) - 1
+            i = idxs[last]
+            pick.append(i)
+            suppress = [last]
+
+            # loop over all indexes in the indexes list
+            for pos in range(0, last):
+                # grab the current index
+                j = idxs[pos]
+
+                # find the largest (x, y) coordinates for the start of
+                # the bounding box and the smallest (x, y) coordinates
+                # for the end of the bounding box
+                xx1 = max(x1[i], x1[j])
+                yy1 = max(y1[i], y1[j])
+                xx2 = min(x2[i], x2[j])
+                yy2 = min(y2[i], y2[j])
+
+                # compute the width and height of the bounding box
+                w = max(0, xx2 - xx1 + 1)
+                h = max(0, yy2 - yy1 + 1)
+
+                # compute the ratio of overlap between the computed
+                # bounding box and the bounding box in the area list
+                overlap = float(w * h) / area[j]
+
+                # if there is sufficient overlap, suppress the
+                # current bounding box
+                if overlap >threshold:
+                    suppress.append(pos)
+
+            # delete all indexes from the index list that are in the
+            # suppression list
+            idxs = np.delete(idxs, suppress)
+
+        # return only the bounding boxes that were picked
+        return boxes[pick],pick
 
     def infer_single_image(self, img, scale=None):
         if self._model.built:
-            self._model.to(self.device)
-            self._model.eval()
-            img = image2array(img)
-            if img.shape[-1] == 4:
-                img = img[:, :, :3]
-            img_orig = img.copy()
-
-            for func in self.preprocess_flow:
-                if inspect.isfunction(func):
-                    img = func(img)
-                    if func.__qualname__ == 'resize.<locals>.img_op':
-                        scale = func.scale
-
-            img = image_backend_adaptive(img)
-            inp = to_tensor(np.expand_dims(img, 0)).to(
-                torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu")).to(
-                self._model.weights[0].data.dtype)
-            confidences, boxes = self._model(inp)
-
-            boxes, labels, probs = self.predict(640, 480, to_numpy(confidences), to_numpy(boxes),
-                                                prob_threshold=self.detection_threshold,
-                                                iou_threshold=self.iou_threshold)
-            fn = rescale(1 / scale[0])
             try:
-                img, boxes = fn(img[0], boxes)
+                self._model.to(self.device)
+                self._model.eval()
+                img = image2array(img)
+                if img.shape[-1] == 4:
+                    img = img[:, :, :3]
+                img_orig = img.copy()
+
+                for func in self.preprocess_flow:
+                    if inspect.isfunction(func):
+                        img = func(img)
+                        if func.__qualname__ == 'resize.<locals>.img_op':
+                            scale = func.scale
+
+                img = image_backend_adaptive(img)
+                inp = to_tensor(np.expand_dims(img, 0)).to(
+                    torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu")).to(
+                    self._model.weights[0].data.dtype)
+                confidences_class, boxes,confidences_attr = self._model(inp)
+
+                boxes, labels,confidences_attr= self.predict(640, 480, to_numpy(confidences_class), to_numpy(boxes),to_numpy(confidences_attr),
+                                                    prob_threshold=self.detection_threshold,
+                                                    iou_threshold=0.3)
+                if boxes is not None and len(boxes)>0:
+                    #boxes,keep=self.nms(boxes,threshold=0.3)
+                    boxes,probs=boxes[:,:-1],boxes[:,-1]
+                    boxes=boxes*(1 / scale[0])
+                    #labels=labels[keep].astype(np.uint8)
+                    #fn = rescale(1 / scale[0])
+                    #img, boxes = fn(img[0], boxes)
+                    return img_orig, boxes, labels.astype(np.uint8), probs,confidences_attr
+                else:
+                    return img_orig, None, None, None,None
             except:
                 PrintException()
-            return img_orig, boxes, labels, probs
+
+
         else:
             raise ValueError('the model is not built yet.')
 
     def generate_bboxes(self, *outputs, threshould=0.5, scale=1):
         raise NotImplementedError
 
-    def nms(self, bboxes):
-        raise NotImplementedError
 
 
-def Mobile_RFBnet(base_filters=16, num_classes=4):
+def Mobile_RFBnet(base_filters=16, num_classes=4,num_regressors=4):
     model = SsdDetectionModel(input_shape=(3, 480, 640),
-                              output=RFBnet(base_filters=base_filters, num_classes=num_classes))
+                              output=RFBnet(base_filters=base_filters, num_classes=num_classes,num_regressors=num_regressors))
     model.signature = get_signature(model.model.forward)
     model.preprocess_flow = [resize((480, 640), keep_aspect=True, align_corner=True), normalize(127.5, 127.5)]
     return model
