@@ -370,7 +370,7 @@ class Model(ModelBase):
 
     def save_model(self,save_path=None):
         for callback in self.training_context['callbacks']:
-            callback.on_start_save_model(self.training_context)
+            callback.on_model_saving_start(self.training_context)
 
         save_path=save_path if save_path is not None else self.training_context.get('save_path',save_path)
         if save_path is not None:
@@ -383,7 +383,7 @@ class Model(ModelBase):
 
     def save_onnx(self, save_path):
         for callback in self.training_context['callbacks']:
-            callback.on_start_save_model(self.training_context)
+            callback.on_model_saving_start(self.training_context)
         save_path = save_path if save_path is not None else self.training_context.get('save_path', save_path)
         if save_path is not None:
             # torch.save(self.model, file_path)
@@ -461,7 +461,7 @@ class Model(ModelBase):
 
             self.do_post_loss_calculation()
             for callback in self.callbacks:
-                callback.post_loss_calculation(self.training_context)
+                callback.on_loss_calculation_end(self.training_context)
 
             if accumulate_grads==False:
                 #regularizer
