@@ -17,22 +17,51 @@ from ..backend.pytorch_ops import *
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
 _session = get_session()
 
-__all__ = ['MSELoss', 'CrossEntropyLoss', 'BCELoss', 'F1ScoreLoss', 'L1Loss', 'L2Loss', 'make_onehot', 'MS_SSIM',
-           'CrossEntropyLabelSmooth', 'mixup_criterion', 'DiceLoss', 'focal_loss_with_logits', 'FocalLoss',
+__all__ = ['MSELoss', 'CrossEntropyLoss','NullLoss','BCELoss', 'F1ScoreLoss', 'L1Loss', 'L2Loss', 'make_onehot', 'MS_SSIM',
+           'CrossEntropyLabelSmooth', 'mixup_criterion', 'DiceLoss','IouLoss', 'focal_loss_with_logits', 'FocalLoss',
            'SoftIoULoss', 'CenterLoss', 'TripletLoss', 'LovaszSoftmax', 'PerceptionLoss', 'EdgeLoss',
            'TransformInvariantLoss', 'get_loss']
 
 
-class CrossEntropyLoss(nn.CrossEntropyLoss):
-    def __init__(self, weight=1, ignore_index=-100, reduction='mean', name='CrossEntropyLoss'):
-        super().__init__()
+class CrossEntropyLoss(_Loss):
+    def __init__(self, weight=None,with_logits=True, ignore_index=-100, reduction='mean', name='CrossEntropyLoss'):
+        super().__init__(reduction=reduction)
         self.name = name
-        self.ignore_index = ignore_index
-        self.reduction = reduction
-        self.reduce = True
-        self.size_average = True
+        self.weight=weight
+        self.ignore_index=ignore_index
+        self.with_logits=with_logits
+
     def forward(self, output, target):
-        return super().forward(output, target)
+        if not self.with_logits:
+            output=torch.softmax(output,dim=1)
+        if len(output.size())==len(target.size()):
+            target= argmax(target,1)
+        return F.cross_entropy(output, target,weight=self.weight,ignore_index=self.ignore_index,reduction=self.reduction)
+
+class NullLoss(_Loss):
+    def __init__(self, weight=None, with_logits=True, ignore_index=-100, reduction='mean', name='CrossEntropyLoss'):
+        super().__init__(reduction=reduction)
+        self.name = name
+        self.weight = weight
+        self.ignore_index = ignore_index
+        self.with_logits = with_logits
+
+    def forward(self, output, target):
+        if not self.with_logits:
+            output = torch.log_softmax(output, dim=1)
+        if len(output.size()) == len(target.size()):
+            target = argmax(target, 1)
+        return F.nll_loss(output, target, weight=self.weight, ignore_index=self.ignore_index,
+                               reduction=self.reduction)
+
+    #     if len(output.shape)==2 and len(target.shape)==1:
+    #         return F.cross_entropy(output, target.long(),ignore_index=self.ignore_index,reduction=self.reduction)
+    #     elif len(output.shape)==2 and len(target.shape)==2:
+    #         return super().forward(output, argmax(target,1))
+    #     elif len(output.shape)>2 and len(target.shape)==len(output.shape)-1:
+    #         return super().forward(output.view(-1,output.size(1)), target.long().view(-1))
+    #     elif len(output.shape)>2 and len(target.shape)==len(output.shape):
+    #         return super().forward(output.view(-1,output.size(1)), argmax(target,1).view(-1))
 
 class BCELoss(_Loss):
     def __init__(self, weight=None, reduction='mean', with_logit=False, name='BCELoss'):
@@ -49,7 +78,7 @@ class BCELoss(_Loss):
         target1[target1 == 0] = 1
         target1[target1 != 1] = 0
         target = target.float()
-        if target1.astype(np.float32).mean() == 1:
+        if target1.astype(np.float32).max() == 1:
             if output.shape == target.shape or output.squeeze().shape == target.shape:
                 return F.binary_cross_entropy(output, target, weight=self.loss_weight, reduction=self.reduction)
         elif output.shape[1] == int(target1.max()) and len(output.shape) == len(target.shape) + 1:
@@ -181,24 +210,60 @@ def mixup_criterion(y_a, y_b, lam):
     return lambda criterion, pred: lam * criterion(pred, y_a) + (1 - lam) * criterion(pred, y_b)
 
 
-class DiceLoss(nn.Module):
-    def __init__(self, smooth=1., ignore_index=0):
-        super(DiceLoss, self).__init__()
+
+class DiceLoss(_Loss):
+    def __init__(self,  weight=None, smooth=1., ignore_index=-100,reduction='mean'):
+        super(DiceLoss, self).__init__(reduction=reduction)
         self.ignore_index = ignore_index
         self.smooth = smooth
+        if isinstance(weight,list):
+            weight=to_tensor(np.array(weight))
+        elif isinstance(weight,np.ndarray):
+            weight = to_tensor(weight)
+        if weight is None or isinstance(weight,torch.Tensor) :
+            self.weight=weight
+        else:
+            raise ValueError('weight should be 1-D tensor')
 
     def forward(self, output, target):
-        if self.ignore_index not in range(target.min(), target.max()):
-            if (target == self.ignore_index).sum() > 0:
-                target[target == self.ignore_index] = target.min()
-        target = make_onehot(target.unsqueeze(dim=1), classes=output.size()[1])
-        output = F.softmax(output, dim=1)
-        output_flat = output.contiguous().view(-1)
-        target_flat = target.contiguous().view(-1)
-        intersection = (output_flat * target_flat).sum()
-        loss = 1 - ((2. * intersection + self.smooth) / (output_flat.sum() + target_flat.sum() + self.smooth))
-        return loss
+        if self.weight is not None and len(self.weight)!=output.size(1):
+            raise ValueError('weight should be 1-D tensor and length equal to numbers of filters')
 
+        target = make_onehot(target, classes=output.size()[1])
+        probs = F.softmax(output, dim=1)
+        #probs=output#*((output>0.5).float())
+        if self.reduction=='mean':
+            intersection = (target * probs).sum(-1).sum(-1)
+            den1=probs.sum(-1).sum(-1)
+            den2= target.sum(-1).sum(-1)
+            dice =1-((2 *intersection+ self.smooth) / (den1 + den2+self.smooth))
+            if self.weight is not None:
+                dice=dice*(self.weight.unsqueeze(0))
+            dice1 = 1 - ((2 * intersection[:, 1:].sum() + self.smooth) / (den1[:, 1:].sum() + den2[:, 1:].sum() + self.smooth))
+            return dice[:, 1:].mean()#+dice1#.sum()
+        else:
+            intersection = (target * probs)[:, 1:].sum()
+            den1 = probs[:, 1:].sum()
+            den2 = target[:, 1:].sum()
+            dice = 1 - ((2 * intersection + self.smooth) / (den1 + den2 + self.smooth))
+            return dice
+
+class IouLoss(_Loss):
+    def __init__(self, ignore_index=-1000,reduction='mean'):
+        super(IouLoss, self).__init__(reduction=reduction)
+        self.ignore_index = ignore_index
+    def forward(self, output, target):
+        output = argmax(output, 1)
+        output_flat = output.reshape(-1)
+        target_flat = target.reshape(-1)
+        output_flat = output_flat[target_flat != self.ignore_index]
+        target_flat = target_flat[target_flat != self.ignore_index]
+        output_flat = output_flat[target_flat != 0]
+        target_flat = target_flat[target_flat != 0]
+        intersection =(output_flat == target_flat).float().sum()
+        union = ((output_flat + target_flat) > 0).float().sum().clamp(min=1)
+        loss = -(intersection/union).log()
+        return loss
 
 def focal_loss_with_logits(input: torch.Tensor, target: torch.Tensor, gamma=2.0, alpha=0.25, reduction="mean",
         normalized=False, threshold=None, ) -> torch.Tensor:
@@ -222,6 +287,18 @@ def focal_loss_with_logits(input: torch.Tensor, target: torch.Tensor, gamma=2.0,
 
         https://github.com/open-mmlab/mmdetection/blob/master/mmdet/core/loss/losses.py
     """
+    if len(target.shape) == len(input.shape):
+        target = target.type(input.type())
+    elif len(target.shape) + 1 == len(input.shape) and target.shape[0] == input.shape[0]:
+        num_classes = None
+        if input.shape[1:-1] == target.shape[1:]:
+            num_classes = input.shape[-1]
+        elif input.shape[2:] == target.shape[1:]:
+            num_classes = input.shape[1]
+        target = make_onehot(target, num_classes)
+        if target.shape != input.shape:
+            target = target.reshape(input.shape)
+
     target = target.type(input.type())
 
     logpt = -F.binary_cross_entropy_with_logits(input, target, reduction="none")
@@ -269,6 +346,7 @@ class FocalLoss(_Loss):
         loss = 0
 
         # Filter anchors with -1 label from loss computation
+        not_ignored=target
         if self.ignore_index is not None:
             not_ignored = target != self.ignore_index
 
@@ -279,10 +357,9 @@ class FocalLoss(_Loss):
             if self.ignore_index is not None:
                 cls_target = cls_target[not_ignored]
                 cls_input = cls_input[not_ignored]
+            loss+=focal_loss_with_logits(cls_input, cls_target, gamma=self.gamma, alpha=self.alpha, reduction=self.reduction)
 
-            loss = loss + focal_loss_with_logits(cls_input, cls_target, gamma=self.gamma, alpha=self.alpha,
-                                                 reduction=self.reduction)
-        return loss.sum()
+        return loss/output.size(0) if self.reduction=='mean' else loss
 
 
 class SoftIoULoss(_Loss):
