@@ -9,9 +9,12 @@ from torch.autograd import Variable
 from functools import partial
 import numpy as np
 from ..backend.common import get_session,addindent,get_time_suffix,get_class,get_function,camel2snake
-from ..backend.pytorch_ops import argmax
+from ..backend.pytorch_backend import to_numpy
+from ..backend.pytorch_ops import *
+from ..data.mask_common import mask2trimap
 
-__all__ = ['accuracy','pixel_accuracy','iou','psnr','mean_absolute_error','mean_squared_error','mean_squared_logarithmic_error','mae','mse','rmse','msle','get_metric']
+
+__all__ = ['accuracy','pixel_accuracy','alpha_pixel_accuracy','iou','psnr','mean_absolute_error','mean_squared_error','mean_squared_logarithmic_error','mae','mse','rmse','msle','get_metric']
 
 # def accuracy(input, target,axis=1):
 #     input_tensor=input.clone().detach()
@@ -132,6 +135,24 @@ def pixel_accuracy(output, target):
     pixel_correct = ((input_tensor == target_tensor)*(target_tensor > 0)).sum().float()
     return pixel_correct/max(pixel_labeled,1)
 
+def alpha_pixel_accuracy(output, alpha):
+    output_tensor = to_numpy(output)
+    alpha_tensor =  to_numpy(alpha)
+
+    trimap=alpha_tensor.copy()
+    trimap[(0<trimap)*(trimap<1)==True]=0.5
+    if len(output_tensor.shape)==len(alpha_tensor.shape)+1 and output_tensor.shape[1]==2:
+        # trimap_out=mask2trimap()(np.argmax(output_tensor,1))
+        # trimap_out1=trimap_out.copy()
+        # trimap_out1[trimap_out1==0.5]=0
+        # trimap_out2=trimap_out.copy()
+        # trimap_out2[trimap_out2 ==1] = 0
+        output_tensor=output_tensor[:,1,:,:]*np.argmax(output_tensor,1)#trimap_out*trimap_out1+output_tensor[:,1,:,:]*trimap_out2
+        output_tensor[output_tensor>0.95]=1
+    pixel_labeled = (output_tensor > 0).sum()
+    pixel_correct = ((output_tensor == alpha_tensor)*(trimap == 1)).sum()+ (np.less(np.abs(output_tensor - alpha_tensor),0.1).astype(np.float32)*(trimap == 0.5)).sum()
+    return pixel_correct/max(pixel_labeled,1)
+
 def iou(output, target):
     input_tensor = output.clone().detach()
     target_tensor = target.clone().detach()
