@@ -8,6 +8,7 @@ import math
 import torch
 import torch.nn as nn
 import torch.optim as optim
+
 from torch.optim.optimizer import  Optimizer
 import torch.nn.functional as F
 from torch.autograd import Variable
@@ -20,14 +21,19 @@ from ..backend.pytorch_backend import *
 from .trainers import OptimizerMixin
 
 
-__all__ = ['Adam','SGD','Adadelta','Adagrad','RMSprop','RAdam','PlainRAdam','AdamW','Lookahead','Ranger','get_optimizer']
+__all__ = ['Adam','SGD','LBFGS','Adadelta','Adagrad','RMSprop','RAdam','PlainRAdam','AdamW','Lookahead','Ranger','get_optimizer']
 
 class Adam(optim.Adam,OptimizerMixin):
     pass
 
+
+
 class SGD(optim.SGD,OptimizerMixin):
     pass
 
+class LBFGS(get_class('LBFGS',['torch.optim']),OptimizerMixin):
+    pass
+
 class Adadelta(get_class('Adadelta',['torch.optim']),OptimizerMixin):
     pass
 
