@@ -49,9 +49,10 @@ def orth_reg(model:nn.Module,reg_weight=1e-6):
             loss = loss + (reg_weight * sym.abs().sum())
     return loss
 
-def total_variation_norm_reg(output:torch.Tensor,reg_weight=1e-6):
+def total_variation_norm_reg(output:torch.Tensor,reg_weight=1):
+    total_variation_norm_reg.reg_weight=reg_weight
     assert len(output.size())==4
-    loss = reg_weight * (torch.sum(torch.abs(output[:, :, :, :-1] - output[:, :, :, 1:])) + torch.sum(torch.abs(output[:, :, :-1, :] - output[:, :, 1:, :])))
+    loss = reg_weight * (torch.sum(torch.abs(output[:, :, :, :-1] - output[:, :, :, 1:])) + torch.sum(torch.abs(output[:, :, :-1, :] - output[:, :, 1:, :])))/2.0
     return loss
 
 
