@@ -1,11 +1,5 @@
 import os
-import sys
-import matplotlib
-import platform
-import string
 import copy
-# if platform.system() not in ['Linux', 'Darwin'] and not platform.system().startswith('CYGWIN'):
-#     matplotlib.use('TKAgg')
 from ..misc.ipython_utils import is_in_ipython
 
 if is_in_ipython():
@@ -17,7 +11,7 @@ import string
 import uuid
 import torch
 import torch.nn as nn
-
+import shutil
 from functools import partial
 import numpy as np
 from ..backend.common import *
@@ -79,8 +73,11 @@ class Model(ModelBase):
     def _initial_graph(self, inputs=None, output=None, input_shape=None):
         if output is None:
             raise ValueError('There is at least one output')
+        if isinstance(output,(np.ndarray,torch.Tensor)) and input_shape is None:
+            input_shape=output.shape
 
         if inputs is None:
+
             if input_shape is None:
                 raise ValueError('You should assign inputs or input shape')
             else:
@@ -100,6 +97,7 @@ class Model(ModelBase):
             for k, v in inputs.items():
                 if isinstance(v, Input):
                     self.inputs[k] = to_numpy(v.input_shape).tolist()
+
         if isinstance(output, (Layer, nn.Module)):
             output.input_shape = input_shape
             # output.cpu()
@@ -108,14 +106,14 @@ class Model(ModelBase):
             self._model = output
 
             if isinstance(out, torch.Tensor):
-                self.outputs['output'] = list(out.size())[1:]
-                self.targets['target'] = list(out.size())[1:]
+                self._outputs['output'] = to_list(out.size())[1:]
+                self._targets['target'] = to_list(out.size())[1:]
             else:
                 for i in range(len(out)):
-                    self.outputs['output_{0}'.format(i)] = list(out[i].size())[1:]
-                    self.targets['target_{0}'.format(i)] = list(out[i].size())[1:]
+                    self._outputs['output_{0}'.format(i)] = to_list(out[i].size())[1:]
+                    self._targets['target_{0}'.format(i)] = to_list(out[i].size())[1:]
 
-            self.signature =get_signature(self._model.forward)
+            self.signature = get_signature(self._model.forward)
         elif isinstance(output, (list, tuple)):
             output_list = []
             for op in output:
@@ -125,41 +123,83 @@ class Model(ModelBase):
             model = Combine(output_list)
             outs = model(dummay_input)
             self._model = model
+            self.name = model.name
             for i in range(len(outs)):
-                self.outputs['output_{0}'.format(i)] = list(outs[i].size())[1:]
-                self.targets['target_{0}'.format(i)] = list(outs[i].size())[1:]
-            self.signature =get_signature(self._model.forward)
+                self._outputs['output_{0}'.format(i)] = to_list(outs[i].size())[1:]
+                self._targets['target_{0}'.format(i)] = to_list(outs[i].size())[1:]
+            self.signature = get_signature(self._model.forward)
+        elif isinstance(output,(np.ndarray,torch.Tensor)):
+            self._model =to_tensor(output,requires_grad=True)
+
+            self._outputs['output'] = to_list(self._model.size())[1:]
+            self._targets['target'] = to_list(self._model.size())[1:]
+            self.signature = OrderedDict()
+            self.signature['x']=to_list(self._model.size())[1:]
         else:
             raise ValueError('Invalid output')
-        for name, module in self._model.named_modules():
-            try:
-                if hasattr(module, '_name'):
-                    if name.isdigit():
-                        module._name = name + '.' + module.name
-                    else:
-                        module._name = name
-            except:
-                PrintException()
+        if not isinstance(output, (np.ndarray, torch.Tensor)):
+            for name, module in self._model.named_modules():
+                try:
+                    if hasattr(module, '_name'):
+                        if name.isdigit():
+                            module._name = name + '.' + module.name
+                        else:
+                            module._name = name
+                except:
+                    PrintException()
 
-        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
-        self._model.to(self.device)
-        self.training_context['current_model'] = self._model
+            self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
+            self._model.to(self.device)
+            self.training_context['current_model'] = self._model
+
+    @property
+    def outputs(self):
+        if self._model is not None and  isinstance(self._model, torch.Tensor):
+            return self._outputs
+        elif self._model is None or not self._model.built:
+            return None
+        elif len(self._outputs) == 1:
+            self._outputs[self._outputs.key_list[0]] = self._model.output_shape.item() if self._model.output_shape.ndim == 0 else to_list(self._model.output_shape)
+            self._targets[self._targets.key_list[0]] = self._model.output_shape.item() if self._model.output_shape.ndim == 0 else to_list(self._model.output_shape)
+
+            return self._outputs
+        elif len(self._outputs) > 1:
+            dummay_input = to_tensor(np.random.standard_normal((1, *self._model.input_shape)).astype(np.float32))
+            outs = self._model(dummay_input)
+            if len(self._outputs) == len(outs):
+                for i in range(len(outs)):
+                    self._outputs[self._outputs.key_list[i]] = to_list(outs[i].size())[1:]
+                    self._targets[self._targets.key_list[i]] = to_list(outs[i].size())[1:]
+            else:
+                for i in range(len(outs)):
+                    self._outputs['output_{0}'.format(i)] = to_list(outs[i].size())[1:]
+                    self._targets['target_{0}'.format(i)] = to_list(outs[i].size())[1:]
+            return self._outputs
+        else:
+            return self._outputs
 
     def train(self):
-        if self._model is not None and self._model.built:
+        if self._model is not None and  isinstance(self._model, torch.Tensor):
+            pass
+        elif self._model is not None and self._model.built:
             self._model.train()
         else:
             raise ValueError('There is no built model ,nothing to learn')
 
     def eval(self):
-        if self._model is not None and self._model.built:
+        if self._model is not None and  isinstance(self._model, torch.Tensor):
+            pass
+        elif self._model is not None and self._model.built:
             self._model.eval()
         else:
             raise ValueError('There is no built model ,nothing to evaluate')
 
     @property
     def layers(self):
-        return self._model._nodes
+        if self._model is not None and  isinstance(self._model, torch.Tensor):
+            return None
+        else:
+            return self._model._nodes
 
     def complie(self, optimizer, losses=None, metrics=None, loss_weights=None, sample_weight_mode=None,
                 weighted_metrics=None, target_tensors=None):
@@ -176,10 +216,10 @@ class Model(ModelBase):
     def with_optimizer(self, optimizer, **kwargs):
         if isinstance(optimizer, str):
             optimizer_class = get_optimizer(optimizer)
-            self.optimizer = optimizer_class(self._model.parameters(), **kwargs)
+            self.optimizer = optimizer_class(self._model.parameters() if  isinstance(self._model, nn.Module) else [self._model], **kwargs)
 
         else:
-            self.optimizer = optimizer(self._model.parameters(), **kwargs)
+            self.optimizer = optimizer(self._model.parameters() if  isinstance(self._model, nn.Module) else [self._model], **kwargs)
         # self.lr_scheduler = torch.optim.lr_scheduler.ReduceLROnPlateau(self.optimizer, verbose=True, mode='min',
         #                                                                factor=0.5, patience=5, threshold=1e-4,
         #                                                                cooldown=0, min_lr=1e-10, eps=1e-8)
@@ -189,66 +229,167 @@ class Model(ModelBase):
         self.training_context['current_lr'] = self.base_lr
         return self
 
-    def with_loss(self, loss, loss_weight=1, output_idx=0, name='', **kwargs):
-        if isinstance(loss, str):
-            loss = get_loss(loss)
+    def with_loss(self, loss, loss_weight=1, output_idx=0, start_epoch=0, name='', **kwargs):
         alias = name
-        if inspect.isclass(loss):
-            alias = loss.__name__ if len(alias) == 0 else alias
-        if len(alias) == 0 and hasattr(loss, '__name__'):
+        argnames = OrderedDict()
+        if (alias is None or len(alias) == 0) and hasattr(loss, '__name__'):
             alias = loss.__name__
+
+        if isinstance(loss, str):
+
+            loss_class = get_loss(loss)
+            alias = loss if loss_class is not None else alias
+            if  alias in self._losses:
+                dup_keys=[key for  key in self._losses.key_list if alias+'_' in key]
+                alias = alias + '_' + str(len(dup_keys)+1)
+            self._losses[alias] = loss_class(**kwargs)
+            if hasattr(loss, 'forward'):
+                argnames = get_signature(self._losses[alias].forward)
+            else:
+                argnames = get_signature(self._losses[alias].__call__)
+        elif inspect.isclass(loss) and inspect._is_type(loss):
+            alias = loss.__class__.__name__ if alias is None or len(alias) == 0 else alias
+            if alias in self._losses:
+                dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
+                alias = alias + '_' + str(len(dup_keys)+1)
+
+            self._losses[alias] = loss(**kwargs)
+            if hasattr(loss, 'forward'):
+                argnames = get_signature(self._losses[alias].forward)
+            else:
+                argnames = get_signature(self._losses[alias].__call__)
+        elif not inspect.isfunction(loss) and callable(loss):
+            alias = loss.__class__.__name__ if len(alias) == 0 else alias
+            if alias in self._losses:
+                dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
+                alias = alias + '_' + str(len(dup_keys) + 1)
+            self._losses[alias] = loss
+            if hasattr(loss, 'forward'):
+                argnames = get_signature(self._losses[alias].forward)
+            else:
+                argnames = get_signature(self._losses[alias].__call__)
+        elif inspect.isfunction(loss):
+            if alias in self._losses:
+                dup_keys = [key for key in self._losses.key_list if alias + '_' in key]
+                alias = alias + '_' + str(len(dup_keys) + 1)
+            spec = inspect.getfullargspec(loss)
+            if len(spec.args) >= 2 and len(spec.args) - 0 if spec.defaults is None else len(spec.defaults) == 2 and (
+                    spec.args[0] in ['output', 'y_pred', 'pred'] or 'target_' + spec.args[0] == spec.args[1]):
+                self._losses[alias] = loss
+            else:
+                self._losses[alias] = partial(loss, **kwargs)
+            argnames = get_signature(self._losses[alias], skip_default=True)
+
         self.loss_weights[alias] = loss_weight
 
-        argnames=[]
-        if hasattr(loss, 'forward'):
-            self._losses[alias] = loss(**kwargs)
-            if hasattr(self._losses[alias], 'reduction'):
-                setattr(self._losses[alias], 'reduction', 'mean')
-            if hasattr(self._losses[alias], 'size_average'):
-                setattr(self._losses[alias], 'size_average', 'True')
-            argnames=get_signature(self._losses[alias].forward)
-        elif callable(loss):
-            if inspect.isfunction(loss):
-                spec = inspect.getfullargspec(loss)
-                if len(spec.args) >= 2 and len(spec.args) - 0 if spec.defaults is None else len(spec.defaults) == 2 and  (spec.args[0] in ['output',  'y_pred','pred'] or  'target_'+spec.args[0]==spec.args[1]):
-                    self._losses[alias] = loss
-                else:
-                    self._losses[alias] = partial(loss, **kwargs)
-            argnames = get_signature(self._losses[alias])
-
-        output_idx=list(output_idx) if isinstance(output_idx,(list,tuple)) else [output_idx]
-        if len(output_idx) == 1:
-            out = self.outputs.key_list[output_idx[0]]
-            target = self.targets.key_list[output_idx[0]]
-
+        outputs = self.outputs
+        targets = self.targets
+        if outputs is not None and len(outputs) == 1 and len(argnames) == 2 and argnames.key_list[0] in ['input','output','y_pred'] and  argnames.key_list[1] in ['target','label','y_true']:
+            argnames = OrderedDict()
+            argnames[outputs.key_list[0]] = outputs[outputs.key_list[0]]
+            argnames[targets.key_list[0]] = targets[targets.key_list[0]]
+        elif outputs is not None and len(outputs) == 1 and len(argnames) == 2 :
+            argnames[argnames.key_list[0]] = outputs[outputs.key_list[0]]
+            argnames[argnames.key_list[1]] = targets[targets.key_list[0]]
+        elif outputs is not None and len(outputs) > 1:
+            output_idx = list(output_idx) if isinstance(output_idx, (list, tuple)) else [output_idx]
+            if len(output_idx) == 1 and len(argnames) == 2:
+                argnames = OrderedDict()
+                out = outputs.key_list[output_idx[0]]
+                target = targets.key_list[output_idx[0]]
+                argnames[argnames.key_list[0]] = outputs[out]
+                argnames[argnames.key_list[1]] = targets[target]
+            elif len(output_idx) > 1 and len(argnames) == 2 * len(output_idx):
+                for idx in output_idx:
+                    out = outputs.key_list[idx]
+                    target = targets.key_list[idx]
+                    if out in argnames:
+                        argnames[out] = outputs[out]
+                    if target in argnames:
+                        argnames[target] = targets[target]
+        self._losses[alias].__name__ = alias
         self._losses[alias].signature = argnames
-        print('{0} signature:{1}'.format(alias, self._losses[alias].signature.key_list))
+        self._losses[alias].start_epoch = start_epoch
+        print('{0} signature:{1}'.format(alias, self._losses[alias].signature.item_list))
         return self
 
     def with_metric(self, metric, output_idx=0, name='', **kwargs):
-        if isinstance(metric, str):
-            metric = get_metric(metric)
-
         alias = name
-        if inspect.isfunction(metric):
-            alias = metric.__name__ if len(alias) == 0 else alias
-        if len(alias) == 0 and hasattr(metric, 'name'):
-            alias = metric.name
-        if hasattr(metric, 'forward'):
-            self._metrics[alias] = metric( **kwargs)  # if hasattr(self._metrics[metric.__name__], 'reduction'):  #     setattr(self._metrics[metric.__name__], 'reduction', 'mean')
-        elif callable(metric):
-            self._metrics[alias] = partial(metric, **kwargs)
-        argnames = get_signature(self._metrics[alias])
-        output_idx = list(output_idx) if isinstance(output_idx, (list, tuple)) else [output_idx]
-
-        if len(output_idx)==1:
-            out = self.outputs.key_list[output_idx[0]]
-            target = self.targets.key_list[output_idx[0]]
-
-        self._metrics[alias].signature  = argnames
-        print('{0} signature:{1}'.format(alias, self._metrics[alias].signature.key_list))
+        argnames = OrderedDict()
+        if (alias is None or len(alias) == 0) and hasattr(metric, '__name__'):
+            alias = metric.__name__
 
+        if isinstance(metric, str):
+            alias = metric if len(alias) == 0 else alias
+            metric_class = get_metric(metric)
+            if alias in self._metrics:
+                dup_keys = [key for key in self._metrics.key_list if alias + '_' in key]
+                alias = alias + '_' + str(len(dup_keys) + 1)
+            self._metrics[alias] = metric_class(**kwargs)
+            if hasattr(metric, 'forward'):
+                argnames = get_signature(self._metrics[alias].forward)
+            else:
+                argnames = get_signature(self._metrics[alias].__call__)
+        elif inspect.isclass(metric) and inspect._is_type(metric):
+            alias = metric.__class__.__name__ if len(alias) == 0 else alias
+            if alias in self._metrics:
+                dup_keys = [key for key in self._metrics.key_list if alias + '_' in key]
+                alias = alias + '_' + str(len(dup_keys) + 1)
+            self._metrics[alias] = metric(**kwargs)
+            if hasattr(metric, 'forward'):
+                argnames = get_signature(self._metrics[alias].forward)
+            else:
+                argnames = get_signature(self._metrics[alias].__call__)
+        elif not inspect.isfunction(metric)and callable(metric):
+            alias = metric.__class__.__name__ if len(alias) == 0 else alias
+            if alias in self._metrics:
+                dup_keys = [key for key in self._metrics.key_list if alias + '_' in key]
+                alias = alias + '_' + str(len(dup_keys) + 1)
+            self._metrics[alias] = metric
+            if hasattr(metric, 'forward'):
+                argnames = get_signature(self._metrics[alias].forward)
+            else:
+                argnames = get_signature(self._metrics[alias].__call__)
+        elif inspect.isfunction(metric):
+            if alias in self._metrics:
+                dup_keys = [key for key in self._metrics.key_list if alias + '_' in key]
+                alias = alias + '_' + str(len(dup_keys) + 1)
+            spec = inspect.getfullargspec(metric)
+            if len(spec.args) >= 2 and len(spec.args) - 0 if spec.defaults is None else len(spec.defaults) == 2 and (
+                    spec.args[0] in ['output', 'y_pred', 'pred'] or 'target_' + spec.args[0] == spec.args[1]):
+                self._metrics[alias] = metric
+            else:
+                self._metrics[alias] = partial(metric, **kwargs)
+            argnames = get_signature(self._metrics[alias], skip_default=True)
 
+        outputs = self.outputs
+        targets = self.targets
+        if outputs is not None and len(outputs) == 1 and len(argnames) == 2 and argnames.key_list[0] in ['input', 'output', 'y_pred'] and argnames.key_list[1] in ['target', 'label', 'y_true']:
+            argnames = OrderedDict()
+            argnames[outputs.key_list[0]] = outputs[outputs.key_list[0]]
+            argnames[targets.key_list[0]] = targets[targets.key_list[0]]
+        elif outputs is not None and len(outputs) == 1 and len(argnames) == 2:
+            argnames[argnames.key_list[0]] = outputs[outputs.key_list[0]]
+            argnames[argnames.key_list[1]] = targets[targets.key_list[0]]
+        elif outputs is not None and len(outputs) > 1:
+            output_idx = list(output_idx) if isinstance(output_idx, (list, tuple)) else [output_idx]
+            if len(output_idx) == 1 and len(argnames) == 2:
+                argnames = OrderedDict()
+                out = outputs.key_list[output_idx[0]]
+                target = targets.key_list[output_idx[0]]
+                argnames[argnames.key_list[0]] = outputs[out]
+                argnames[argnames.key_list[1]] = targets[target]
+            elif len(output_idx) > 1 and len(argnames) == 2 * len(output_idx):
+                for idx in output_idx:
+                    out = outputs.key_list[idx]
+                    target = targets.key_list[idx]
+                    if out in argnames:
+                        argnames[out] = outputs[out]
+                    if target in argnames:
+                        argnames[target] = targets[target]
+        self._metrics[alias].__name__ = alias
+        self._metrics[alias].signature = argnames
+        print('{0} signature:{1}'.format(alias, self._metrics[alias].signature.item_list))
         return self
 
     def with_regularizer(self, reg, **kwargs):
@@ -259,12 +400,16 @@ class Model(ModelBase):
             reg_fn = get_reg(reg)
         elif reg is callable:
             reg_fn = reg
-        args = reg_fn.__code__.co_varnames
+        args = get_signature(reg_fn)
         if 'reg_weight' in args:
-            if 'model' in args:
-                self._model_regs[reg_fn.__name__] = partial(reg_fn, **kwargs)
-            elif 'output' in args:
-                self._output_regs[reg_fn.__name__] = partial(reg_fn, **kwargs)
+            args.pop('reg_weight')
+
+        if 'model' in args:
+            self._model_regs[reg_fn.__name__] = partial(reg_fn, **kwargs)
+            self._model_regs[reg_fn.__name__].signature = args
+        elif 'output' in args:
+            self._output_regs[reg_fn.__name__] = partial(reg_fn, **kwargs)
+            self._output_regs[reg_fn.__name__].signature = args
         return self
 
     def with_constraint(self, constraint, **kwargs):
@@ -297,8 +442,8 @@ class Model(ModelBase):
             self.callbacks.append(lr_scheduler)
         self.warmup = warmup
         if self.warmup > 0:
-            self.optimizer.adjust_learning_rate(1e-5, False)
-            self.training_context['current_lr'] = 1e-5
+            self.optimizer.adjust_learning_rate(1e-6, False)
+            self.training_context['current_lr'] = 1e-6
         return self
 
     def adjust_learning_rate(self, lr):
@@ -306,7 +451,9 @@ class Model(ModelBase):
         self.training_context['current_lr'] = lr
 
     def rebinding_input_output(self, input_shape):
-        if input_shape == self._model._input_shape:
+        if self._model is not None and  isinstance(self._model, torch.Tensor):
+            pass
+        elif input_shape == self._model._input_shape:
             pass
         else:
             if len(self.inputs) == 1:
@@ -321,17 +468,22 @@ class Model(ModelBase):
             if isinstance(out, torch.Tensor) == len(self.targets) == 1:
                 self.targets[self.targets.key_list[0]] = list(out.size())[1:] if len(out.size()) > 1 else []
                 self._model.output_shape = out.size()[1:]
-            elif isinstance(out, tuple) :
+            elif isinstance(out, tuple):
                 for k in range(len(out)):
                     self.targets[self.targets.key_list[k]] = list(out[k].size())[1:] if len(out[k].size()) > 1 else []
 
-
     def do_on_training_start(self):
-        self._model.train()
-        self._model.zero_grad()
+        if self._model is not None and  isinstance(self._model, torch.Tensor):
+            pass
+        else:
+            self._model.train()
+            self._model.zero_grad()
 
     def do_on_training_end(self):
-        self._model.eval()
+        if self._model is not None and  isinstance(self._model, torch.Tensor):
+            pass
+        else:
+            self._model.eval()
 
     def do_on_epoch_start(self):
         # if self.training_context['current_epoch'] < self.warmup:
@@ -341,7 +493,7 @@ class Model(ModelBase):
         # elif self.training_context['current_epoch'] == self.warmup:
         #     self.optimizer.param_groups[0]['lr'] = self.base_lr
         #     self.training_context['current_lr'] =self.base_lr
-        if self.device == 'cuda':
+        if self.model.device == 'cuda':
             torch.cuda.empty_cache()
 
     def do_on_epoch_end(self):
@@ -351,10 +503,10 @@ class Model(ModelBase):
         pass
 
     def do_on_batch_end(self):
-        if self.training_context['current_batch'] == 0 :
-            temp=OrderedDict()
+        if self.training_context['current_batch'] == 0:
+            temp = OrderedDict()
             for k in self.training_context['losses'].key_list:
-                temp[k]=self.training_context['losses'][k][-1]
+                temp[k] = self.training_context['losses'][k][-1]
             print(temp)
 
     def do_on_data_received(self, train_data, test_data):
@@ -370,7 +522,6 @@ class Model(ModelBase):
                 available_fields = copy.deepcopy(train_data.key_list)
                 if train_data is not None:
                     # check input
-
                     for arg in self.signature.key_list:
                         data_feed[arg] = ''
                     for arg in self.signature.key_list:
@@ -380,12 +531,15 @@ class Model(ModelBase):
                         elif arg in available_fields:
                             data_feed[arg] = arg
                             available_fields.remove(arg)
-                        elif arg=='x' and 'data' in available_fields:
+                        elif arg in ['x', 'input'] and 'data' in available_fields:
                             data_feed[arg] = 'data'
-                            available_fields.remove( 'data' )
-                        elif arg=='x' and 'input' in available_fields:
+                            available_fields.remove('data')
+                        elif arg in ['x', 'input'] and 'image' in available_fields:
+                            data_feed[arg] = 'image'
+                            available_fields.remove('image')
+                        elif arg == 'x' and 'input' in available_fields:
                             data_feed[arg] = 'input'
-                            available_fields.remove( 'input' )
+                            available_fields.remove('input')
                         elif len(self.signature.key_list) == 1:
                             for item in available_fields:
                                 data_shape = list(train_data[item].shape[1:]) if len(train_data[item].shape) > 1 else []
@@ -398,6 +552,9 @@ class Model(ModelBase):
                                 'input argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
                                     arg))
 
+                    if len(self.signature.key_list) == 1 and data_feed[self.signature.key_list[0]] != None:
+                        self.training_context['data_feed'] = data_feed
+
                     # check for target
                     for i in range(len(self.targets)):
                         arg = self.targets.key_list[i]
@@ -407,12 +564,12 @@ class Model(ModelBase):
                         elif arg in available_fields:
                             data_feed[arg] = arg
                             available_fields.remove(arg)
-                        elif arg=='target' and 'label' in available_fields:
+                        elif arg == 'target' and 'label' in available_fields:
                             data_feed[arg] = 'label'
-                            available_fields.remove( 'label' )
-                        elif arg=='target' and len(available_fields)==1:
-                            data_feed[arg] =available_fields[0]
-                            available_fields.remove( available_fields[0])
+                            available_fields.remove('label')
+                        elif arg == 'target' and len(available_fields) == 1:
+                            data_feed[arg] = available_fields[0]
+                            available_fields.remove(available_fields[0])
                         elif len(available_fields) > 0:
                             target_shape = outshapes[i]
                             for item in available_fields:
@@ -428,25 +585,36 @@ class Model(ModelBase):
                                     Warning(
                                         'target argment {0} cannot mapping to any data, please check it and update the datafeed'.format(
                                             arg))
+                    if len(self.targets) == 1 and data_feed[self.targets.key_list[0]] != None:
+                        self.training_context['current_target'] = train_data[data_feed[self.targets.key_list[0]]]
 
                     self.training_context['data_feed'] = data_feed
+                    print('data_feed', data_feed)
             except:
                 PrintException()
 
         # convert to tensor
         try:
-            data_feed=self.training_context['data_feed']
-            input_list=[data_feed[arg] for arg in self.signature.key_list]
+            data_feed = self.training_context['data_feed']
+            input_list = [data_feed[arg] for arg in self.signature.key_list]
             for item in train_data.key_list:
                 if item in input_list:
-                    #only model 's input argments
-                    train_data[item] = to_tensor(train_data[item],requires_grad=True)
+                    # only model 's input argments
+                    train_data[item] = to_tensor(train_data[item].copy(), requires_grad=True)
+                elif item in self.targets.key_list or data_feed:
+                    train_data[item] = to_tensor(train_data[item].copy())
                 else:
-                    train_data[item] = to_tensor(train_data[item])
+                    train_data[item] = to_tensor(train_data[item].copy())
+
                 if test_data is not None:
-                    test_data[item] = to_tensor(test_data[item])
+                    test_data[item] = to_tensor(test_data[item].copy())
 
                     # check target
+
+            self.training_context['train_data'] = train_data
+            if test_data is not None:
+                self.training_context['test_data'] = test_data
+
         except:
             PrintException()
         return train_data, test_data
@@ -503,14 +671,16 @@ class Model(ModelBase):
 
     def save_model(self, save_path=None):
         for callback in self.training_context['callbacks']:
-            callback.on_start_save_model(self.training_context)
+            callback.on_model_saving_start(self.training_context)
         self._model.eval()
         if save_path is None or len(save_path) == 0:
             save_path = self.training_context.get('save_path', self.save_path)
             if save_path is None or len(save_path) == 0:
                 save_path = 'Models/model_{0}_epoch{1}.pth'.format(self._model.name,
                                                                    self.training_context['current_epoch'])
-        torch.save({'state_dict': self._model.state_dict()}, save_path + '.tar')
+        torch.save({'state_dict': self._model.state_dict()}, save_path + '.tar_')
+        # save safety
+        shutil.copyfile(save_path + '.tar_', save_path + '.tar')
         self._model.train()
 
     def save_onnx(self, file_path, dynamic_axes=None):
@@ -536,7 +706,7 @@ class Model(ModelBase):
                           dummy_input,  # model input (or a tuple for multiple inputs)
                           file_path,  # where to save the model (can be a file or file-like object)
                           export_params=True,  # store the trained parameter weights inside the model file
-                          opset_version=10,  # the ONNX version to export the model to
+                          opset_version=11,  # the ONNX version to export the model to
                           do_constant_folding=True,  # whether to execute constant folding for optimization
                           input_names=self.inputs.key_list,  # the model's input names
                           output_names=output_names,  # the model's output names
@@ -547,7 +717,6 @@ class Model(ModelBase):
         print('Loading pretrained model from {}'.format(file_path))
         pretrained_dict = torch.load(file_path, map_location=lambda storage, loc: storage)
 
-
         if "state_dict" in pretrained_dict.keys():
             pretrained_dict = pretrained_dict['state_dict']
         # else:
@@ -558,10 +727,10 @@ class Model(ModelBase):
 
         if self.signature is None:
             self.signature = get_signature(self._model.forward)
-
+        self._model.to(torch.device("cuda" if torch.cuda.is_available() else "cpu"))
 
     def summary(self):
-        #self.rebinding_input_output(self._model.input_shape)
+        # self.rebinding_input_output(self._model.input_shape)
         summary(self._model, tuple(self._model.input_shape))
 
     def extra_repr(self):
@@ -684,17 +853,19 @@ class ImageClassificationModel(Model):
                 self._model.weights[0].data.dtype)
             result = self._model(inp)
             result = to_numpy(result)[0]
+            if self.class_names is None or len(self.class_names)==0:
+                return result
+            else:
+                # argresult = np.argsort(result)
+                # argresult1 =argresult[::-1]
+                answer = OrderedDict()
+                idxs = list(np.argsort(result)[::-1][:topk])
+                for idx in idxs:
+                    prob = result[idx]
+                    answer[self.index2label(idx)] = (idx, prob)
+                # idx=int(np.argmax(result,-1)[0])
 
-            # argresult = np.argsort(result)
-            # argresult1 =argresult[::-1]
-            answer = OrderedDict()
-            idxs = list(np.argsort(result)[::-1][:topk])
-            for idx in idxs:
-                prob = result[idx]
-                answer[self.index2label(idx)] = (idx, prob)
-            # idx=int(np.argmax(result,-1)[0])
-
-            return answer
+                return answer
         else:
             raise ValueError('the model is not built yet.')
 
@@ -726,7 +897,9 @@ class ImageDetectionModel(Model):
                 if inspect.isfunction(func):
                     img = func(img)
             img = image_backend_adaptive(img)
-            inp = to_tensor(np.expand_dims(img, 0)).to(torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu")).to(self._model.weights[0].data.dtype)
+            inp = to_tensor(np.expand_dims(img, 0)).to(
+                torch.device("cuda" if self._model.weights[0].data.is_cuda else "cpu")).to(
+                self._model.weights[0].data.dtype)
             result = self._model(inp)
 
             bboxes = self.generate_bboxes(*result, threshould=self.detection_threshould, scale=scale)
@@ -797,6 +970,3 @@ TrainingItem = Model
 
 
 
-
-
-
