@@ -397,7 +397,7 @@ class Model(ModelBase):
 
     def save_model(self, save_path=None):
         for callback in self.training_context['callbacks']:
-            callback.on_start_save_model(self.training_context)
+            callback.on_model_saving_start(self.training_context)
         save_path = self.training_context.get('save_path', save_path)
         if save_path is not None:
             self._model.save(save_path)
@@ -416,7 +416,7 @@ class Model(ModelBase):
 
     def save_weights(self, file_path):
         for callback in self.training_context['callbacks']:
-            callback.on_start_save_model(self.training_context)
+            callback.on_model_saving_start(self.training_context)
 
         if file_path is not None:
             self._model.save_weights(file_path)
@@ -546,7 +546,7 @@ class Model(ModelBase):
                     self.training_context['optimizer'] = self.optimizer
                     self.do_post_loss_calculation()
                     for callback in self.callbacks:
-                        callback.post_loss_calculation(self.training_context)
+                        callback.on_loss_calculation_end(self.training_context)
 
                     grads = g.gradient(self.training_context['current_loss'], self._model.trainable_variables)
 
