@@ -1,7 +1,7 @@
 import os
 import sys
 import tkinter
-from collections import namedtuple
+
 
 import matplotlib
 import platform
@@ -198,8 +198,8 @@ class OptimizerMixin(object):
 class ModelBase(object):
     def __init__(self, inputs=None, output=None, input_shape=None, name='', **kwargs):
         self.inputs = OrderedDict()
-        self.outputs = OrderedDict()
-        self.targets = OrderedDict()
+        self._outputs = OrderedDict()
+        self._targets = OrderedDict()
         self._model = None
         self.name = name
         self.optimizer = None
@@ -261,25 +261,41 @@ class ModelBase(object):
                 self.inputs = OrderedDict()
                 self.targets = OrderedDict()
                 self._initial_graph(output=value, input_shape=inp_shape)
-                args = [param.name for param in inspect.signature(self._model.forward).parameters.values()]
-                self._signature = namedtuple('signature', args)
+                self._signature =get_signature(value.forward)
             else:
                 self.inputs = OrderedDict()
                 self.targets = OrderedDict()
                 if self._model is not None:
                     self._model.cpu()
                 self._initial_graph(output=value, input_shape=value.input_shape)
-                args = [param.name for param in inspect.signature(self._model.forward).parameters.values()]
-                self._signature = namedtuple('signature', args)
+                self._signature = get_signature(value.forward)
         elif isinstance(value, nn.Module):
             self._model = value
-            args = [param.name for param in inspect.signature(self._model.forward).parameters.values()]
-            self._signature = namedtuple('signature', args)
+            self._signature = get_signature(value.forward)
         elif isinstance(value, np.ndarray) or 'tensor' in value.__name__.lower():
             self._model = to_tensor(value)
         else:
             raise ValueError('Only Layer, Module, Image and Tensor can be valid model')
 
+
+    @property
+    def outputs(self):
+        return self._outputs
+
+    @outputs.setter
+    def outputs(self, value):
+        self._outputs=value
+
+
+    @property
+    def targets(self):
+        return self._targets
+
+    @targets.setter
+    def targets(self, value):
+        self._targets=value
+
+
     @property
     def batch_metric_history(self):
         return self.training_context['metrics']
@@ -332,19 +348,21 @@ class ModelBase(object):
             new_outputs = OrderedDict()
             new_target = OrderedDict()
             outputs_args = arg_names[len(self.signature.key_list):]
+            outputs=self._outputs
+            targets=self._targets
             for i in range(len(outputs_args)):
                 arg = outputs_args[i]
-                new_outputs[arg] = self.outputs.value_list[0]
+                new_outputs[arg] = outputs.value_list[0]
 
                 target_arg = arg.replace('output', 'target')
                 if 'target' not in target_arg:
                     target_arg = 'target_' + target_arg
-                new_target[target_arg] = self.targets.value_list[0]
-            self.outputs = new_outputs
-            self.targets = new_target
+                new_target[target_arg] = targets.value_list[0]
+            self._outputs = new_outputs
+            self._targets = new_target
             print(self._signature.key_list)
-            print(self.outputs.key_list)
-            print(self.targets.key_list)
+            print(self._outputs.key_list)
+            print(self._targets.key_list)
         elif not isinstance(arg_names, (list, tuple)):
             raise ValueError('arg_names should be list or tuple')
         elif len(self.signature.key_list) < len(arg_names) and len(self.signature.key_list) + len(self.outputs) == len(
@@ -373,7 +391,7 @@ class ModelBase(object):
         elif len(self._signature.key_list) != len(arg_names):
             raise ValueError('data deed and arg_names should be the same length')
         else:
-            self.signature = namedtuple('signature', arg_names)
+            #self.signature = namedtuple('signature', arg_names)
             print(self.signature.key_list)
 
     def _initial_graph(self, inputs=None, output=None, input_shape=None):
@@ -567,6 +585,9 @@ class ModelBase(object):
             self.training_context['log_weights'] = log_weights
             self.training_context['current_model'] = self._model
             self.training_context['current_lr'] = self.optimizer.lr
+            self.training_context['train_data'] = train_data
+            self.training_context['test_data'] = test_data
+
 
             self.sample_collect_history.append(1 if is_collect_data else 0)
 
@@ -591,6 +612,8 @@ class ModelBase(object):
                     callback.on_epoch_start(self.training_context)
 
             self.do_on_batch_start()
+            for callback in self.callbacks:
+                callback.on_batch_start(self.training_context)
 
             train_data, test_data = self.do_on_data_received(train_data, test_data)
 
@@ -604,54 +627,67 @@ class ModelBase(object):
 
             output = try_map_args_and_call(self._model, train_data, self.training_context['data_feed'])
             # write output in to data
+
             if isinstance(output, (list, tuple)):
                 for i in range(len(output)):
                     train_data[self.outputs.key_list[i]] = output[i]
+            elif 'tensor' in output.__class__.__name__.lower():
+                train_data[self.outputs.key_list[0]] =output
             else:
                 train_data[self.outputs.key_list[0]] = output
 
+
             # confirm singleton
             # output=unpack_singleton(output)
 
             # losss
             for k, v in self._losses.items():
-                if k not in self.training_context['losses']:
-                    self.training_context['losses'][k] = []
-                try:
-                    loss_weight = 1
-                    if k in self.loss_weights:
-                        loss_weight = self.loss_weights[k]
-                    this_loss = try_map_args_and_call(v, train_data, self.training_context[
-                        'data_feed']) * loss_weight  # v.forward(output, target) if hasattr(v, 'forward') else v(
-                    # output, target)
+                if not hasattr(v,'start_epoch') or (hasattr(v,'start_epoch') and v.start_epoch<=self.training_context['current_epoch']):
+                    if k not in self.training_context['losses']:
+                        self.training_context['losses'][k] = []
+                    try:
+                        loss_weight = 1
+                        if k in self.loss_weights:
+                            loss_weight = self.loss_weights[k]
+                        this_loss = to_tensor(loss_weight)*try_map_args_and_call(v, train_data, self.training_context['data_feed'])  # v.forward(output, target) if hasattr(v, 'forward') else v(
+                        # output, target)
 
-                    if isinstance(this_loss, tuple):
-                        overall_loss = this_loss[0]
-                        for i in range(1, len(this_loss)):
-                            overall_loss += this_loss[i]
-                        self.training_context['current_loss'] += overall_loss
-                        if is_collect_data:
-                            self.training_context['losses'][k].append(float(to_numpy(overall_loss)))
-                    else:
-                        self.training_context['current_loss'] += this_loss
-                        if is_collect_data:
-                            self.training_context['losses'][k].append(float(to_numpy(this_loss)))
-                except Exception as e:
-                    print(e)
-                    PrintException()
+                        if isinstance(this_loss, tuple):
+                            overall_loss = this_loss[0]
+                            for i in range(1, len(this_loss)):
+                                overall_loss += this_loss[i]
+                            self.training_context['current_loss'] += overall_loss
+                            if is_collect_data:
+                                self.training_context['losses'][k].append(float(to_numpy(overall_loss)))
+                        else:
+                            self.training_context['current_loss'] += this_loss
+                            if is_collect_data:
+                                self.training_context['losses'][k].append(float(to_numpy(this_loss)))
+                    except Exception as e:
+                        print(e)
+                        PrintException()
 
             self.do_post_loss_calculation()
             for callback in self.callbacks:
-                callback.post_loss_calculation(self.training_context)
+                callback.on_loss_calculation_end(self.training_context)
 
             if accumulate_grads == False:
                 # regularizer
                 for k, v in self._output_regs.items():
                     if k + '_Loss' not in self.training_context['losses']:
                         self.training_context['losses'][k + '_Loss'] = []
+
                     this_loss = try_map_args_and_call(v, train_data, self.training_context['data_feed'])
-                    self.training_context['current_loss'] = self.training_context[
-                                                                'current_loss'] + this_loss  # self.training_context[
+                    overall_loss=None
+                    if isinstance(this_loss, tuple):
+                        overall_loss = this_loss[0]
+
+                        for i in range(1, len(this_loss)):
+                            overall_loss += this_loss[i]
+                    else:
+                        overall_loss=this_loss
+
+                    self.training_context['current_loss'] += overall_loss  # self.training_context[
                     # 'current_loss'] + this_loss
                     if is_collect_data:
                         self.training_context['losses'][k + '_Loss'].append(float(to_numpy(this_loss)))
@@ -661,15 +697,23 @@ class ModelBase(object):
                     if k + '_Loss' not in self.training_context['losses']:
                         self.training_context['losses'][k + '_Loss'] = []
                     this_loss = v(self._model)
-                    self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
+                    overall_loss = None
+                    if isinstance(this_loss, tuple):
+                        overall_loss = this_loss[0]
+
+                        for i in range(1, len(this_loss)):
+                            overall_loss += this_loss[i]
+                    else:
+                        overall_loss = this_loss
+                    self.training_context['current_loss'] = self.training_context['current_loss'] + overall_loss
                     if is_collect_data:
-                        self.training_context['losses'][k + '_Loss'].append(float(to_numpy(this_loss)))
+                        self.training_context['losses'][k + '_Loss'].append(float(to_numpy(overall_loss)))
 
                 self.training_context['optimizer'] = self.optimizer
                 # self.do_post_loss_calculation()
                 #
                 # for callback in self.callbacks:
-                #     callback.post_loss_calculation(self.training_context)
+                #     callback.on_loss_calculation_end(self.training_context)
 
                 self.do_pre_optimization_step()
                 self.do_gradient_update(log_gradients and is_collect_data)
@@ -704,10 +748,16 @@ class ModelBase(object):
                         self.training_context['tmp_metrics'][k] = []
                         self.training_context['metrics'][k] = []
 
-                    this_metric = try_map_args_and_call(v, test_data if test_data is not None and len(
-                        test_data) > 0 else train_data, self.training_context['data_feed'])
+                    this_metric = try_map_args_and_call(v, test_data if test_data is not None and len(test_data) > 0 else train_data, self.training_context['data_feed'])
 
                     self.training_context['tmp_metrics'][k].append(to_numpy(this_metric).mean())
+
+                # ON_EVALUATION_END
+                self.do_on_metrics_evaluation_end()
+                for callback in self.training_context['callbacks']:
+                    callback.on_metrics_evaluation_end(self.training_context)
+
+                #callback's metric can keep in epoch_metric_history
                 for k, v in self.training_context['tmp_metrics'].items():
                     if k not in self.epoch_metric_history:
                         self.epoch_metric_history[k] = []
@@ -717,11 +767,6 @@ class ModelBase(object):
                         self.training_context['metrics'][k].append(float(to_numpy(v).mean()))
                         self.training_context['tmp_metrics'][k] = []
 
-                # ON_EVALUATION_END
-                self.do_on_metrics_evaluation_end()
-                for callback in self.training_context['callbacks']:
-                    callback.on_metrics_evaluation_end(self.training_context)
-
                 # print batch progresss
                 if is_print_batch_progress:
                     self.do_on_progress_start()
@@ -894,14 +939,23 @@ class TrainingPlan(object):
     def add_training_item(self, training_item, name=''):
         n = len(self.training_items)
 
+        alias=name if len(name)>0 else  training_item.name
+        alias = alias if len(alias) > 0 else training_item.model.name
+        alias = alias if len(alias) > 0 else  'model {0}'.format(n)
+
+
         if len(training_item.name) > 0:
             self.training_names[n] = training_item.name
         else:
             if len(name) > 0:
                 training_item.name = name
+                if isinstance(training_item.model,nn.Module):
+                    training_item.model.name=name
                 self.training_names[n] = name
             else:
                 training_item.name = 'model {0}'.format(n)
+                if isinstance(training_item.model, nn.Module):
+                    training_item.model.name ='model {0}'.format(n)
                 self.training_names[n] = 'model {0}'.format(n)
         self.training_items[n] = training_item
         return self
@@ -975,15 +1029,57 @@ class TrainingPlan(object):
         self.callbacks.append(plot)
         return self
 
+    def generate_datafeed(self):
+        for trainingitem in self.training_items.value_list:
+            data_feed=OrderedDict()
+            outputs=trainingitem.outputs
+            targets=trainingitem.targets
+            if len(self._dataloaders)==1:
+                data_loader = self._dataloaders.value_list[0]
+                if data_loader.signature is None:
+                    _=data_loader.next()
+                if len(trainingitem.model.signature)+len(targets)==len(data_loader.signature):
+                    if len(outputs)==1 and len(trainingitem.model.signature)==1  :
+
+                        if  1<=len(data_loader.signature)<=2:
+                                data_feed[trainingitem.model.signature.key_list[0]]=data_loader.signature.key_list[0]
+                                for loss in trainingitem._losses.value_list:
+                                    args=get_signature(loss)
+                                    if hasattr(loss,'signature'):
+                                        args=loss.signature
+                                    if len(args)==2:
+                                        if args.key_list[0] not in data_feed or (args.key_list[0] in data_feed and data_feed[args.key_list[0]] is None):
+                                            data_feed[args.key_list[0]]=outputs.key_list[0]
+                                        if args.key_list[1] not in data_feed or (args.key_list[1] in data_feed and data_feed[args.key_list[1]] is None):
+                                            if args.key_list[1] in data_loader.signature.key_list:
+                                                data_feed[args.key_list[1]] = args.key_list[1]
+                                            else:
+                                                data_feed[args.key_list[1]] =data_loader.signature.key_list[-1]   #-1 is for handel autoencoder scenario
+                                    else:
+                                        raise ValueError('loss shoud only 2 argments when one-input-one-output model with 2 dataset items in data loaders')
+
+                                trainingitem.training_context['data_feed']=data_feed
+                                print('data_feed for {0} :{1}'.format(trainingitem.name,data_feed))
+
+
+                else:
+                    raise RuntimeError('the number of models input plus the numbers of  targets should equal to the numbers of dataset items')
+            else:
+                raise  RuntimeError('Multiple data loader data_feed auto-generation is not support Now.')
+
+
+
+
+
+
+
     def start_now(self, collect_data_inteval=1, is_resume=False):
         try:
-            self.execution_id = str(uuid.uuid4())[:8].__str__().replace('-',
-                                                                        '') if self.execution_id is None else self.execution_id
-
+            self.execution_id = str(uuid.uuid4())[:8].__str__().replace('-', '') if self.execution_id is None else self.execution_id
             # update callback
             if not is_resume:
+                self.generate_datafeed()
                 for item in self.training_items.values():
-                    item.rebinding_input_output(item.inputs.value_list[0])
                     for callback in self.callbacks:
                         if callback not in item.callbacks:
                             if callback.is_shared == False:
@@ -997,7 +1093,7 @@ class TrainingPlan(object):
             data_loader = self._dataloaders.value_list[0]
             data_loader.minibatch_size = self.minibatch_size
             if not is_resume:
-                print(self.__repr__)
+                #print(self.__repr__)
                 if collect_data_inteval == 1 and len(data_loader.batch_sampler) * self.num_epochs > 1000:
                     collect_data_inteval = self.default_collect_data_inteval
 
@@ -1025,7 +1121,17 @@ class TrainingPlan(object):
                         # input, target = Variable(input).to(self.device), Variable(target).to(self.device)
 
                         for k, trainitem in self.training_items.items():
-                            trainitem.train_model(iter_data, iter_testdata, epoch, mbs, self.num_epochs,
+                            train_data=OrderedDict()
+                            test_data = None if iter_testdata is None else OrderedDict()
+                            for k,v in iter_data.items():
+                                train_data[k]=v.copy()
+
+                            if iter_testdata is not None:
+                                for k,v in iter_testdata.items():
+                                    test_data[k]=v.copy()
+
+
+                            trainitem.train_model(train_data,test_data, epoch, mbs, self.num_epochs,
                                                   len(data_loader.batch_sampler),
                                                   is_collect_data=mbs % collect_data_inteval == 0,
                                                   is_print_batch_progress=self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0,
@@ -1048,8 +1154,7 @@ class TrainingPlan(object):
                         if (mbs + 1) % len(data_loader.batch_sampler) == 0:
                             break
 
-                    if self.save_model_frequency > 0 and self.save_model_unit == 'epoch' and (
-                            epoch + 1) % self.save_model_frequency == 0:
+                    if self.save_model_frequency > 0 and self.save_model_unit == 'epoch' and (epoch + 1) % self.save_model_frequency == 0:
                         for k, trainitem in self.training_items.items():
                             trainitem.save_model()
 
@@ -1175,15 +1280,16 @@ def progress_bar(current, total, msg=None, name=''):
     last_time = cur_time
     tot_time = cur_time - begin_time
     L = []
-    L.append('{0:<16s}'.format(name))
+    L.append('{0:<12s}'.format(name))
     L.append(' Step: {0:<8s}'.format(format_time(step_time)))
     # L.append(' | Tot: {0:<12s}'.format(format_time(tot_time)))
     if msg:
         L.append(' | ' + msg)
     msg = ''.join(L)
     sys.stdout.write(msg)
-    for i in range(term_width - int(TOTAL_BAR_LENGTH) - len(msg) - 3):
-        sys.stdout.write(' ')
+    # for i in range(term_width - int(TOTAL_BAR_LENGTH) - len(msg) - 3):
+    #     sys.stdout.write(' ')
+    sys.stdout.write(' ')
     sys.stdout.write(' ( %d/%d )' % (current, total))
     sys.stdout.write('\n')
     sys.stdout.flush()  # # Go back to the center of the bar.  # for i in range(term_width-int(TOTAL_BAR_LENGTH/2)+2):  #     sys.stdout.write('\b')  # sys.stdout.write(' %d/%d ' % (current+1, total))  # if current < total-1:  #     sys.stdout.write('\r')  # else:  #     sys.stdout.write('\n')  # sys.stdout.flush()
